@@ -196,6 +196,7 @@ main (argc, argv)
   char gwdirb[BUFSIZ];
   char *cwd;
   char *temp;
+  char *progname = argv[0];
 
   if (argc < 2)
     {
@@ -251,19 +252,20 @@ main (argc, argv)
   while (argc)
     {
       int need_cwd = 0;
-      if (*argv[0] == '+')
+      char *modified_arg = argv[0];
+      if (*modified_arg == '+')
 	{
-	  char *p = argv[0] + 1;
+	  char *p = modified_arg + 1;
 	  while (*p >= '0' && *p <= '9') p++;
 	  if (*p != 0)
 	    need_cwd = 1;
 	}
-      else if (*argv[0] != '/')
+      else if (*modified_arg != '/')
 	need_cwd = 1;
 
       if (need_cwd)
 	used += strlen (cwd);
-      used += strlen (argv[0]) + 1;
+      used += strlen (modified_arg) + 1;
       while (used + 2 > size_allocated)
 	{
 	  size_allocated *= 2;
@@ -275,7 +277,7 @@ main (argc, argv)
       if (need_cwd)
 	strcat (msgp->mtext, cwd);
 
-      strcat (msgp->mtext, argv[0]);
+      strcat (msgp->mtext, modified_arg);
       strcat (msgp->mtext, " ");
       argv++; argc--;
     }
@@ -290,7 +292,7 @@ main (argc, argv)
   msgp->mtype = 1;
   if (msgsnd (s, msgp, strlen (msgp->mtext)+1, 0) < 0)
     {
-      fprintf (stderr, "%s: ", argv[0]);
+      fprintf (stderr, "%s: ", progname);
       perror ("msgsnd");
       exit (1);
     }
