@@ -5,7 +5,7 @@
 ;; Author: Ake Stenhoff <etxaksf@aom.ericsson.se>
 ;;         Lars Lindberg <lli@sypro.cap.se>
 ;; Created: 8 Feb 1994
-;; Version: 1.7
+;; Version: 1.11
 ;; Keywords: tools
 ;;
 ;; This program is free software; you can redistribute it and/or modify
@@ -43,6 +43,15 @@
 ;;   supplied.
 
 ;;; Change Log:
+;;    v1.11 Jul 26 1994 Ake Stenhoff
+;;      Fixed bugs in 'imenu-add-to-menubar'.
+;;    v1.10 Jul 21 1994 Ake Stenhoff
+;;      Added support for markers.
+;;      Changed the examples to use
+;;      markers.
+;;      Thanks [alon].
+;;    v1.9 Jun 14 1994 Ake Stenhoff
+;;      Added 'imenu-add-to-menubar'.
 ;;    v1.7 Apr 12 1994 Ake Stenhoff
 ;;	Changed doc strings refering to symbols.
 ;;      Require 'cl' when compiling only.
@@ -77,7 +86,7 @@
 ;;; Thanks goes to
 ;;  [simon] - Simon Leinen simon@lia.di.epfl.ch
 ;;  [dean] - Dean Andrews ada@unison.com
-;;  
+;;  [alon] - Alon Albert al@mercury.co.il 
 
 ;;; Code
 (eval-when-compile (require 'cl))
@@ -115,7 +124,7 @@ element should come before the second.  The arguments are cons cells;
 
 (defvar imenu-scanning-message "Scanning buffer for index. (%3d%%)"
   "*Progress message during the index scanning of the buffer.
-If non NIL, user gets a message during the scanning of the buffer
+If non-nil, user gets a message during the scanning of the buffer
 
 Relevant only if the mode-specific function that creates the buffer
 index use `imenu-progress-message'.")
@@ -213,18 +222,22 @@ This function is called after the function pointed out by
       (/ (* 100 (1- pos)) (max total 1)))))
 
 ;;;
-;;; Macro to display a progress message.  This will probably be used
-;;; in a tight loop, that is why we use a macro.
+;;; Macro to display a progress message.
 ;;; RELPOS is the relative position to display.
 ;;; If RELPOS is nil, then the relative position in the buffer
 ;;; is calculated.
-(defmacro imenu-progress-message (&optional relpos reverse)
+;;; PREVPOS is the variable in which we store the last position displayed.
+(defmacro imenu-progress-message (prevpos &optional relpos reverse)
   (` (and
       imenu-scanning-message
-      (message imenu-scanning-message
-	       (, (if relpos
+      (let ((pos (, (if relpos
 		      relpos
-		    (` (imenu--relative-position (, reverse)))))))))
+		    (` (imenu--relative-position (, reverse)))))))
+	(if (, (if relpos t
+		 (` (> pos (+ 5 (, prevpos))))))
+	    (progn
+	      (message imenu-scanning-message pos)
+	      (setq (, prevpos) pos)))))))
 
 ;;;
 ;;; Function for suporting general looking submenu names.
@@ -283,7 +296,25 @@ This function is called after the function pointed out by
       (error "No items suitable for an index found in this buffer."))
   ;; Add a rescan option to the index.
   (cons imenu--rescan-item imenu--index-alist))
-
+;;;
+;;; Find all markers in alist and makes
+;;; them point nowhere.
+;;;
+(defun imenu--cleanup (&optional alist)
+  ;; Sets the markers in imenu--index-alist 
+  ;; point nowhere.
+  ;; if alist is provided use that list.
+  (and imenu--index-alist
+       (mapcar 
+	(function
+	 (lambda (item)
+	   (cond
+	    ((markerp (cdr item))
+	     (set-marker (cdr item) nil))
+	    ((listp (cdr item))
+	     (imenu--cleanup (cdr item))))))
+	 (if alist alist imenu--index-alist))
+	t))
 (defun imenu-default-create-index-function ()
   "*Wrapper for index searching functions.
 
@@ -306,17 +337,17 @@ Their results are gathered into an index alist."
 	     (error "The mode \"%s\" does not take full advantage of imenu.el yet."
 		    mode-name))      
 	 (let ((index-alist '())
-	       name)
+	       name prev-pos)
 	   (goto-char (point-max))
-	   (imenu-progress-message 0 t)
+	   (imenu-progress-message prev-pos 0 t)
 	   ;; Search for the function
 	   (while (funcall imenu-prev-index-position-function) 
-	     (imenu-progress-message nil t)
+	     (imenu-progress-message prev-pos nil t)
 	     (save-excursion
 	       (setq name (funcall imenu-extract-index-name-function)))
 	     (and (stringp name)
 		  (push (cons name (point)) index-alist)))
-	   (imenu-progress-message 100 t)
+	   (imenu-progress-message prev-pos 100 t)
 	   index-alist))))
 
 (defun imenu--replace-spaces (name replacement)
@@ -346,7 +377,7 @@ Their results are gathered into an index alist."
 				  (concat prefix imenu-level-separator name)
 				name))))
 	(cond
-	 ((numberp pos)
+	 ((or (markerp pos) (numberp pos))
 	  (list (cons new-prefix pos)))
 	 (t
 	  (imenu--flatten-index-alist pos new-prefix))))))
@@ -448,12 +479,12 @@ not.
 
 The returned value is on the form (INDEX-NAME . INDEX-POSITION)."
   (let (index-alist
-	(mouse-triggered (listp last-command-event))
+	(mouse-triggered (listp last-nonmenu-event))
 	(result t) )
     ;; If selected by mouse, see to that the window where the mouse is
     ;; really is selected.
     (and mouse-triggered
-	 (let ((window (posn-window (event-start last-command-event))))
+	 (let ((window (posn-window (event-start last-nonmenu-event))))
 	   (or (framep window) (select-window window))))
     ;; Create a list for this buffer only when needed.
     (while (eq result t)
@@ -461,12 +492,23 @@ The returned value is on the form (INDEX-NAME . INDEX-POSITION)."
       (setq result
 	    (if (and mouse-triggered
 		     (not imenu-always-use-completion-buffer-p))
-		(imenu--mouse-menu index-alist last-command-event)
+		(imenu--mouse-menu index-alist last-nonmenu-event)
 	      (imenu--completion-buffer index-alist prompt)))
       (and (eq result t)
+	   (imenu--cleanup)
 	   (setq imenu--index-alist nil)))
     result))
 
+(defun imenu-add-to-menubar (name)
+  "Adds an \"imenu\" entry to the menubar for the 
+current local keymap.
+NAME is the string naming the menu to be added.
+See 'imenu' for more information."
+  (interactive "sMenu name: ")
+  (and window-system
+       (define-key (current-local-map) [menu-bar index]
+	 (cons name 'imenu))))
+
 ;;;###autoload
 (defun imenu ()
   "Jump to a place in the buffer chosen using a buffer menu or mouse menu.
@@ -476,7 +518,12 @@ See `imenu-choose-buffer-index' for more information."
     (and index-item
 	 (progn
 	   (push-mark)
-	   (goto-char (cdr index-item))))))
+	   (cond
+	    ((markerp (cdr index-item))
+	     (goto-char (marker-position (cdr index-item))))
+	    (t
+	     (goto-char (cdr index-item))))))))
+
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;;
@@ -485,15 +532,17 @@ See `imenu-choose-buffer-index' for more information."
 ;;;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-;; Return the current/previous sexp and the location of the sexp (it's
+;; Return the current/previous sexp and the location of the sexp (its
 ;; beginning) without moving the point.
 (defun imenu-example--name-and-position ()
   (save-excursion
     (forward-sexp -1)
     (let ((beg (point))
-	  (end (progn (forward-sexp) (point))))
+	  (end (progn (forward-sexp) (point)))
+	  (marker (make-marker)))
+      (set-marker marker beg)
       (cons (buffer-substring beg end)
-	    beg))))
+	    marker))))
   
 ;;;
 ;;; Lisp
@@ -519,12 +568,13 @@ See `imenu-choose-buffer-index' for more information."
   (let ((index-alist '())
 	(index-var-alist '())
 	(index-type-alist '())
-	(index-unknown-alist '()))
+	(index-unknown-alist '())
+	prev-pos)
     (goto-char (point-max))
-    (imenu-progress-message 0)
+    (imenu-progress-message prev-pos 0)
     ;; Search for the function
     (while (beginning-of-defun)
-      (imenu-progress-message nil t)
+      (imenu-progress-message prev-pos nil t)
       (save-match-data
 	(and (looking-at "(def")
 	     (save-excursion
@@ -551,7 +601,7 @@ See `imenu-choose-buffer-index' for more information."
 		 (forward-sexp 2)
 		 (push (imenu-example--name-and-position)
 		       index-unknown-alist)))))))
-    (imenu-progress-message 100)
+    (imenu-progress-message prev-pos 100)
     (and index-var-alist
 	 (push (cons (imenu-create-submenu-name "Variables") index-var-alist)
 	       index-alist))
@@ -579,15 +629,15 @@ See `imenu-choose-buffer-index' for more information."
 
 (defun imenu-example--create-c-index (&optional regexp)
   (let ((index-alist '())
-	(char))
+	prev-pos char)
     (goto-char (point-min))
-    (imenu-progress-message 0)
+    (imenu-progress-message prev-pos 0)
     ;; Search for the function
     (save-match-data
       (while (re-search-forward
 	      (or regexp imenu-example--function-name-regexp-c)
 	      nil t)
-	(imenu-progress-message)
+	(imenu-progress-message prev-pos)
 	(backward-up-list 1)
 	(save-excursion
 	  (goto-char (scan-sexps (point) 1))
@@ -595,7 +645,7 @@ See `imenu-choose-buffer-index' for more information."
 	;; Skip this function name if it is a prototype declaration.
 	(if (not (eq char ?\;))
 	    (push (imenu-example--name-and-position) index-alist))))
-    (imenu-progress-message 100)
+    (imenu-progress-message prev-pos 100)
     (nreverse index-alist)))
 
 ;;; 
