@@ -6,9 +6,6 @@
 ;;;
 ;;;
 ;;; Authors         : Ken Stevens et. al.
-;;; Last Modified By: Ken Stevens <k.stevens@ieee.org>
-;;; Last Modified On: Fri May 20 15:58:52 MDT 1994
-;;; Update Revision : 2.30
 ;;; Syntax          : emacs-lisp
 ;;; Status	    : Release with 3.1.05 ispell.
 ;;; Version	    : International Ispell Version 3.1 by Geoff Kuenning.
@@ -408,11 +405,12 @@ For example, '(\"-W\" \"3\") to cause it to accept all 1-3 character
 words as correct.  See also `ispell-dictionary-alist', which may be used
 for language-specific arguments.")
 
-(defvar ispell-dictionary-alist		; sk  9-Aug-1991 18:28
+;;;###autoload
+(defvar ispell-dictionary-alist-1	; sk  9-Aug-1991 18:28
   '((nil				; default (english.aff)
-     "[A-Za-z]" "[^A-Za-z]" "[---']" nil ("-B") nil)
+     "[A-Za-z]" "[^A-Za-z]" "[']" nil ("-B") nil)
     ("english"				; make english explicitly selectable
-     "[A-Za-z]" "[^A-Za-z]" "[---']" nil ("-B") nil)
+     "[A-Za-z]" "[^A-Za-z]" "[']" nil ("-B") nil)
     ("deutsch"				; deutsch.aff
      "[a-zA-Z\"]" "[^a-zA-Z\"]" "[---']" t ("-C") nil)
     ("deutsch8"
@@ -422,22 +420,31 @@ for language-specific arguments.")
     ("nederlands8"				; dutch8.aff
      "[A-Za-z\300-\305\307\310-\317\322-\326\331-\334\340-\345\347\350-\357\361\362-\366\371-\374]"
      "[^A-Za-z\300-\305\307\310-\317\322-\326\331-\334\340-\345\347\350-\357\361\362-\366\371-\374]"
-     "[---']" t ("-C") nil)
-    ("svenska"				;7 bit swedish mode
+     "[---']" t ("-C") nil)))
+
+;;;###autoload
+(defvar ispell-dictionary-alist-2
+  '(("svenska"				;7 bit swedish mode
      "[A-Za-z}{|\\133\\135\\\\]" "[^A-Za-z}{|\\133\\135\\\\]"
      "[---']" nil ("-C") nil)
     ("svenska8"				;8 bit swedish mode
      "[A-Za-z\345\344\366\305\304\366]"  "[^A-Za-z\345\344\366\305\304\366]"
      "[---']" nil ("-C" "-d" "svenska") "~list") ; Add `"-T" "list"' instead?
     ("francais"
-     "[A-Za-z]" "[^A-Za-z]" "[`'^---]" t nil nil)
+     "[A-Za-z]" "[^A-Za-z]" "[`'^-]" t nil nil)
     ("francais8" "[A-Za-z\300\302\306\307\310\311\312\313\316\317\324\331\333\334\340\342\346\347\350\351\352\353\356\357\364\371\373\374]"
      "[^A-Za-z\300\302\304\306\307\310\311\312\313\316\317\324\326\331\333\334\340\342\344\346\347\350\351\352\353\356\357\364\366\371\373\374]" "[---']"
      t nil "~list")
     ("dansk"				; dansk.aff
      "[A-Z\306\330\305a-z\346\370\345]" "[^A-Z\306\330\305a-z\346\370\345]"
      "[---]" nil ("-C") nil)
-    )
+    ))
+
+;; ispell-dictionary-alist is set up from two subvariables above
+;; to avoid having very long lines in loaddefs.el.
+;;;###autoload
+(defvar ispell-dictionary-alist
+  (append ispell-dictionary-alist-1 ispell-dictionary-alist-2)
   "An alist of dictionaries and their associated parameters.
 
 Each element of this list is also a list:
@@ -479,7 +486,68 @@ Note that the CASECHARS and OTHERCHARS slots of the alist should
 contain the same character set as casechars and otherchars in the
 language.aff file \(e.g., english.aff\).")
 
+;;;###autoload
+(defvar ispell-menu-map nil)
 
+;;; Set up the map.
+;;;###autoload
+(defconst ispell-menu-map-needed
+  ;; Verify this is not Lucid Emacs.
+  (and (not ispell-menu-map) (boundp 'system-key-alist)))
+
+;;;###autoload
+(if ispell-menu-map-needed
+    (let ((dicts (reverse (cons (cons "default" nil) ispell-dictionary-alist)))
+	  name)
+      ;; Can put in defvar when external defines are removed.
+      (setq ispell-menu-map (make-sparse-keymap "Spell"))
+      (while dicts
+	(setq name (car (car dicts))
+	      dicts (cdr dicts))
+	(if (stringp name)
+	    (define-key ispell-menu-map (vector (intern name))
+	      (cons (concat "Select " (capitalize name))
+		    (list 'lambda () '(interactive)
+			  (list 'ispell-change-dictionary name))))))))
+
+;;;###autoload
+(if ispell-menu-map-needed
+    (progn
+      ;; Define commands in opposite order you want them to appear in menu.
+      (define-key ispell-menu-map [ispell-change-dictionary]
+	'("Change Dictionary" . ispell-change-dictionary))
+      (define-key ispell-menu-map [ispell-kill-ispell]
+	'("Kill Process" . ispell-kill-ispell))
+      (define-key ispell-menu-map [ispell-pdict-save]
+	'("Save Dictionary" . (lambda () (interactive) (ispell-pdict-save t))))
+      (define-key ispell-menu-map [ispell-complete-word]
+	'("Complete Word" . ispell-complete-word))
+      (define-key ispell-menu-map [ispell-complete-word-interior-frag]
+	'("Complete Word Frag" . ispell-complete-word-interior-frag))))
+
+;;;###autoload
+(if ispell-menu-map-needed
+    (progn
+      (define-key ispell-menu-map [ispell-continue]
+	'("Continue Check" . ispell-continue))
+      (define-key ispell-menu-map [ispell-region]
+	'("Check Region" . ispell-region))
+      (define-key ispell-menu-map [ispell-word]
+	'("Check Word" . ispell-word))
+      (define-key ispell-menu-map [ispell-buffer]
+	'("Check Buffer" . ispell-buffer))))
+
+;;;###autoload
+(if ispell-menu-map-needed
+    (progn
+      (define-key ispell-menu-map [ispell-message]
+	'("Check Message" . ispell-message))
+      (define-key ispell-menu-map [ispell-help]
+	'("Help" . (lambda () (interactive) (describe-function 'ispell-help))))
+      (put 'ispell-region 'menu-enable 'mark-active)))
+
+;;;###autoload
+(fset 'ispell-menu-map (symbol-value 'ispell-menu-map))
 
 (cond
  ((and (string-lessp "19" emacs-version)
@@ -514,53 +582,7 @@ language.aff file \(e.g., english.aff\).")
     (if current-menubar
 	(progn
 	  (delete-menu-item '("Edit" "Spell")) ; in case already defined
-	  (add-menu '("Edit") "Spell" ispell-menu-lucid)))))
-
- ;; cond-case:
- ((and (featurep 'menu-bar)
-       (string-lessp "19" emacs-version))
-  (let ((dicts (reverse (cons (cons "default" nil) ispell-dictionary-alist)))
-	name)
-    (defvar ispell-menu-map nil)
-    ;; Can put in defvar when external defines are removed.
-    (setq ispell-menu-map (make-sparse-keymap "Spell"))
-    (while dicts
-      (setq name (car (car dicts))
-	    dicts (cdr dicts))
-      (if (stringp name)
-	  (define-key ispell-menu-map (vector (intern name))
-	    (cons (concat "Select " (capitalize name))
-		  (list 'lambda () '(interactive)
-			(list 'ispell-change-dictionary name))))))
-    ;; Why do we need an alias here?
-    (defalias 'ispell-menu-map ispell-menu-map)
-    ;; Define commands in opposite order you want them to appear in menu.
-    (define-key ispell-menu-map [ispell-change-dictionary]
-      '("Change Dictionary" . ispell-change-dictionary))
-    (define-key ispell-menu-map [ispell-kill-ispell]
-      '("Kill Process" . ispell-kill-ispell))
-    (define-key ispell-menu-map [ispell-pdict-save]
-      '("Save Dictionary" . (lambda () (interactive) (ispell-pdict-save t))))
-    (define-key ispell-menu-map [ispell-complete-word]
-      '("Complete Word" . ispell-complete-word))
-    (define-key ispell-menu-map [ispell-complete-word-interior-frag]
-      '("Complete Word Frag" . ispell-complete-word-interior-frag))
-    (define-key ispell-menu-map [ispell-continue]
-      '("Continue Check" . ispell-continue))
-    (define-key ispell-menu-map [ispell-region]
-      '("Check Region" . ispell-region))
-    (define-key ispell-menu-map [ispell-word]
-      '("Check Word" . ispell-word))
-    (define-key ispell-menu-map [ispell-buffer]
-      '("Check Buffer" . ispell-buffer))
-    (define-key ispell-menu-map [ispell-message]
-      '("Check Message" . ispell-message))
-    (define-key ispell-menu-map [ispell-help]
-      '("Help" . (lambda () (interactive)
-		   (describe-function 'ispell-help)
-		   ;(x-popup-menu last-nonmenu-event(list "" ispell-help-list))
-		   ))))
-  (put 'ispell-region 'menu-enable 'mark-active)))
+	  (add-menu '("Edit") "Spell" ispell-menu-lucid))))))
 
 
 ;;; **********************************************************************
@@ -723,7 +745,6 @@ or \\[ispell-region] to update the Ispell process."
 	      quietly ispell-quietly))
     (ispell-buffer-local-dict)		; use the correct dictionary
     (let ((cursor-location (point))	; retain cursor location
-	  ispell-keep-choices-win	; override global to force creation
 	  (word (ispell-get-word following))
 	  start end poss replace)
       ;; destructure return word info list.
@@ -760,10 +781,11 @@ or \\[ispell-region] to update the Ispell process."
 		 (progn
 		   (if ispell-highlight-p ;highlight word
 		       (ispell-highlight-spelling-error start end t))
-		   (setq replace (ispell-command-loop
-				  (car (cdr (cdr poss)))
-				  (car (cdr (cdr (cdr poss))))
-				  (car poss))))
+		   (save-window-excursion
+		     (setq replace (ispell-command-loop
+				    (car (cdr (cdr poss)))
+				    (car (cdr (cdr (cdr poss))))
+				    (car poss)))))
 	       ;; protected
 	       (if ispell-highlight-p	; clear highlight
 		   (ispell-highlight-spelling-error start end)))
@@ -871,214 +893,206 @@ Returns list for new replacement word (will be rechecked).
 Global `ispell-pdict-modified-p' becomes a list where the only value
 indicates whether the dictionary has been modified when option `a' or `i' is
 used."
-  (unwind-protect
-      (save-window-excursion
-	(let ((count ?0)
-	      (line 2)
-	      (max-lines (- (window-height) 4)) ; assure 4 context lines.
-	      (choices miss)
-	      (window-min-height (min window-min-height
-				      ispell-choices-win-default-height))
-	      (command-characters '( ?  ?i ?a ?A ?r ?R ?? ?x ?X ?q ?l ?u ?m ))
-	      (skipped 0)
-	      char num result)
-	  (save-excursion
-	    (if ispell-keep-choices-win
-		(select-window (previous-window))
-	      (set-buffer (get-buffer-create ispell-choices-buffer))
-	      (setq mode-line-format "--  %b  --"))
-	    (if (equal (get-buffer ispell-choices-buffer) (current-buffer))
-		(erase-buffer)
-	      (error (concat "Bogus, dude! I should be in the *Choices*"
-			     " buffer, but I'm not!")))
-	    (if guess
-		(progn
-		  (insert "Affix rules generate and capitalize "
-			  "this word as shown below:\n\t")
-		  (while guess
-		    (if (> (+ 4 (current-column) (length (car guess)))
-			   (window-width))
-			(progn
-			  (insert "\n\t")
-			  (setq line (1+ line))))
-		    (insert (car guess) "    ")
-		    (setq guess (cdr guess)))
-		  (insert "\nUse option `i' if this is a correct composition"
-			  " from the derivative root.\n")
-		  (setq line (+ line (if choices 3 2)))))
-	    (while (and choices
-			(< (if (> (+ 7 (current-column) (length (car choices))
-				     (if (> count ?~) 3 0))
-				  (window-width))
-			       (progn
-				 (insert "\n")
-				 (setq line (1+ line)))
-			     line)
-			   max-lines))
-	      ;; not so good if there are over 20 or 30 options, but then, if
-	      ;; there are that many you don't want to scan them all anyway...
-	      (while (memq count command-characters) ; skip command characters.
-		(setq count (1+ count)
-		      skipped (1+ skipped)))
-	      (insert "(" count ") " (car choices) "  ")
-	      (setq choices (cdr choices)
-		    count (1+ count)))
-	    (setq count (- count ?0 skipped)))
-
-	  (if ispell-keep-choices-win
-	      (if (> line ispell-keep-choices-win)
+  (let ((count ?0)
+	(line 2)
+	(max-lines (- (window-height) 4)) ; assure 4 context lines.
+	(choices miss)
+	(window-min-height (min window-min-height
+				ispell-choices-win-default-height))
+	(command-characters '( ?  ?i ?a ?A ?r ?R ?? ?x ?X ?q ?l ?u ?m ))
+	(skipped 0)
+	char num result)
+    (save-excursion
+      (set-buffer (get-buffer-create ispell-choices-buffer))
+      (setq mode-line-format "--  %b  --")
+      (erase-buffer)
+      (if guess
+	  (progn
+	    (insert "Affix rules generate and capitalize "
+		    "this word as shown below:\n\t")
+	    (while guess
+	      (if (> (+ 4 (current-column) (length (car guess)))
+		     (window-width))
 		  (progn
-		    (switch-to-buffer ispell-choices-buffer)
-		    (select-window (next-window))
-		    (save-excursion
-		      (let ((cur-point (point)))
-			(move-to-window-line (- line ispell-keep-choices-win))
-			(if (<= (point) cur-point)
-			    (set-window-start (selected-window) (point)))))
-		    (select-window (previous-window))
-		    (enlarge-window (- line ispell-keep-choices-win))
-		    (goto-char (point-min))))
-	    (ispell-overlay-window (max line
-					ispell-choices-win-default-height)))
-	  (switch-to-buffer ispell-choices-buffer)
-	  (goto-char (point-min))
-	  (select-window (next-window))
-	  (while
-	      (eq
-	       t
-	       (setq
-		result
-		(progn
-		  (undo-boundary)
-		  (message (concat "C-h or ? for more options; SPC to leave "
-				   "unchanged, Character to replace word"))
-		  (let ((inhibit-quit t))
-		    (setq char (if (fboundp 'read-char-exclusive)
-				   (read-char-exclusive)
-				 (read-char))
-			  skipped 0)
-		    (if (or quit-flag (= char ?\C-g)) ; C-g is like typing q
-			(setq char ?q
-			      quit-flag nil)))
-		  ;; Adjust num to array offset skipping command characters.
-		  (let ((com-chars command-characters))
-		    (while com-chars
-		      (if (and (> (car com-chars) ?0) (< (car com-chars) char))
-			  (setq skipped (1+ skipped)))
-		      (setq com-chars (cdr com-chars)))
-		    (setq num (- char ?0 skipped)))
+		    (insert "\n\t")
+		    (setq line (1+ line))))
+	      (insert (car guess) "    ")
+	      (setq guess (cdr guess)))
+	    (insert "\nUse option `i' if this is a correct composition"
+		    " from the derivative root.\n")
+	    (setq line (+ line (if choices 3 2)))))
+      (while (and choices
+		  (< (if (> (+ 7 (current-column) (length (car choices))
+			       (if (> count ?~) 3 0))
+			    (window-width))
+			 (progn
+			   (insert "\n")
+			   (setq line (1+ line)))
+		       line)
+		     max-lines))
+	;; not so good if there are over 20 or 30 options, but then, if
+	;; there are that many you don't want to scan them all anyway...
+	(while (memq count command-characters) ; skip command characters.
+	  (setq count (1+ count)
+		skipped (1+ skipped)))
+	(insert "(" count ") " (car choices) "  ")
+	(setq choices (cdr choices)
+	      count (1+ count)))
+      (setq count (- count ?0 skipped)))
 
-		  (cond
-		   ((= char ? ) nil)	; accept word this time only
-		   ((= char ?i)		; accept and insert word into pers dict
-		    (process-send-string ispell-process (concat "*" word "\n"))
-		    (setq ispell-pdict-modified-p '(t)) ; dictionary modified!
-		    nil)
-		   ((or (= char ?a) (= char ?A)) ; accept word without insert
-		    (process-send-string ispell-process (concat "@" word "\n"))
-		    (if (null ispell-pdict-modified-p)
-			(setq ispell-pdict-modified-p
-			      (list ispell-pdict-modified-p)))
-		    (if (= char ?A) 0))	; return 0 for ispell-add buffer-local
-		   ((or (= char ?r) (= char ?R)) ; type in replacement
-		    (if (or (= char ?R) ispell-query-replace-choices)
-			(list (read-string "Query-replacement for: " word) t)
-		      (cons (read-string "Replacement for: " word) nil)))
-		   ((or (= char ??) (= char help-char) (= char ?\C-h))
-		    (ispell-help)
-		    t)
-		   ;; Quit and move point back.
-		   ((= char ?x)
-		    (ispell-pdict-save ispell-silently-savep)
-		    (message "Exited spell-checking")
-		    (setq ispell-quit t)
-		    nil)
-		   ;; Quit and preserve point.
-		   ((= char ?X)
-		    (ispell-pdict-save ispell-silently-savep)
-		    (message
-		     (substitute-command-keys
-		      (concat "Spell-checking suspended;"
-			      " use C-u \\[ispell-word] to resume")))
-		    (setq ispell-quit (max (point-min)
-					   (- (point) (length word))))
-		    nil)
-		   ((= char ?q)
-		    (if (y-or-n-p "Really quit ignoring changes? ")
-			(progn
-			  (ispell-kill-ispell t) ; terminate process.
-			  (setq ispell-quit (or (not ispell-checking-message)
-						(point))
-				ispell-pdict-modified-p nil))
-		      t))		; continue if they don't quit.
-		   ((= char ?l)
-		    (let ((new-word (read-string
-				     "Lookup string (`*' is wildcard): "
-				     word))
-			  (new-line 2))
-		      (if new-word
+    (let ((choices-window (get-buffer-window ispell-choices-buffer)))
+      (if choices-window
+	  (if (not (equal line (window-height choices-window)))
+	      (progn
+		(save-excursion
+		  (let ((cur-point (point)))
+		    (move-to-window-line (- line (window-height choices-window)))
+		    (if (<= (point) cur-point)
+			(set-window-start (selected-window) (point)))))
+		(select-window (previous-window))
+		(enlarge-window (- line (window-height choices-window))))
+	    (select-window choices-window))
+	(ispell-overlay-window (max line
+				    ispell-choices-win-default-height))
+	(switch-to-buffer ispell-choices-buffer)))
+    (goto-char (point-min))
+    (select-window (next-window))
+    (while
+	(eq
+	 t
+	 (setq
+	  result
+	  (progn
+	    (undo-boundary)
+	    (message (concat "C-h or ? for more options; SPC to leave "
+			     "unchanged, Character to replace word"))
+	    (let ((inhibit-quit t))
+	      (setq char (if (fboundp 'read-char-exclusive)
+			     (read-char-exclusive)
+			   (read-char))
+		    skipped 0)
+	      (if (or quit-flag (= char ?\C-g)) ; C-g is like typing X
+		  (setq char ?X
+			quit-flag nil)))
+	    ;; Adjust num to array offset skipping command characters.
+	    (let ((com-chars command-characters))
+	      (while com-chars
+		(if (and (> (car com-chars) ?0) (< (car com-chars) char))
+		    (setq skipped (1+ skipped)))
+		(setq com-chars (cdr com-chars)))
+	      (setq num (- char ?0 skipped)))
+
+	    (cond
+	     ((= char ? ) nil)		; accept word this time only
+	     ((= char ?i)		; accept and insert word into pers dict
+	      (process-send-string ispell-process (concat "*" word "\n"))
+	      (setq ispell-pdict-modified-p '(t)) ; dictionary modified!
+	      nil)
+	     ((or (= char ?a) (= char ?A)) ; accept word without insert
+	      (process-send-string ispell-process (concat "@" word "\n"))
+	      (if (null ispell-pdict-modified-p)
+		  (setq ispell-pdict-modified-p
+			(list ispell-pdict-modified-p)))
+	      (if (= char ?A) 0))	; return 0 for ispell-add buffer-local
+	     ((or (= char ?r) (= char ?R)) ; type in replacement
+	      (if (or (= char ?R) ispell-query-replace-choices)
+		  (list (read-string "Query-replacement for: " word) t)
+		(cons (read-string "Replacement for: " word) nil)))
+	     ((or (= char ??) (= char help-char) (= char ?\C-h))
+	      (ispell-help)
+	      t)
+	     ;; Quit and move point back.
+	     ((= char ?x)
+	      (ispell-pdict-save ispell-silently-savep)
+	      (message "Exited spell-checking")
+	      (setq ispell-quit t)
+	      nil)
+	     ;; Quit and preserve point.
+	     ((= char ?X)
+	      (ispell-pdict-save ispell-silently-savep)
+	      (message
+	       (substitute-command-keys
+		(concat "Spell-checking suspended;"
+			" use C-u \\[ispell-word] to resume")))
+	      (setq ispell-quit (max (point-min)
+				     (- (point) (length word))))
+	      nil)
+	     ((= char ?q)
+	      (if (y-or-n-p "Really kill Ispell process? ")
+		  (progn
+		    (ispell-kill-ispell t) ; terminate process.
+		    (setq ispell-quit (or (not ispell-checking-message)
+					  (point))
+			  ispell-pdict-modified-p nil))
+		t))			; continue if they don't quit.
+	     ((= char ?l)
+	      (let ((new-word (read-string
+			       "Lookup string (`*' is wildcard): "
+			       word))
+		    (new-line 2))
+		(if new-word
+		    (progn
+		      (save-excursion
+			(set-buffer (get-buffer-create
+				     ispell-choices-buffer))
+			(erase-buffer)
+			(setq count ?0
+			      skipped 0
+			      mode-line-format "--  %b  --"
+			      miss (lookup-words new-word)
+			      choices miss)
+			(while (and choices ; adjust choices window.
+				    (< (if (> (+ 7 (current-column)
+						 (length (car choices))
+						 (if (> count ?~) 3 0))
+					      (window-width))
+					   (progn
+					     (insert "\n")
+					     (setq new-line
+						   (1+ new-line)))
+					 new-line)
+				       max-lines))
+			  (while (memq count command-characters)
+			    (setq count (1+ count)
+				  skipped (1+ skipped)))
+			  (insert "(" count ") " (car choices) "  ")
+			  (setq choices (cdr choices)
+				count (1+ count)))
+			(setq count (- count ?0 skipped)))
+		      (select-window (previous-window))
+		      (if (/= new-line line)
 			  (progn
-			    (save-excursion
-			      (set-buffer (get-buffer-create
-					   ispell-choices-buffer))
-			      (erase-buffer)
-			      (setq count ?0
-				    skipped 0
-				    mode-line-format "--  %b  --"
-				    miss (lookup-words new-word)
-				    choices miss)
-			      (while (and choices ; adjust choices window.
-					  (< (if (> (+ 7 (current-column)
-						       (length (car choices))
-						       (if (> count ?~) 3 0))
-						    (window-width))
-						 (progn
-						   (insert "\n")
-						   (setq new-line
-							 (1+ new-line)))
-					       new-line)
-					     max-lines))
-				(while (memq count command-characters)
-				  (setq count (1+ count)
-					skipped (1+ skipped)))
-				(insert "(" count ") " (car choices) "  ")
-				(setq choices (cdr choices)
-				      count (1+ count)))
-			      (setq count (- count ?0 skipped)))
-			    (select-window (previous-window))
-			    (if (/= new-line line)
-				(progn
-				  (if (> new-line line)
-				      (enlarge-window (- new-line line))
-				    (shrink-window (- line new-line)))
-				  (setq line new-line)))
-			    (select-window (next-window)))))
-		    t)			; reselect from new choices
-		   ((= char ?u)
-		    (process-send-string ispell-process
-					 (concat "*" (downcase word) "\n"))
-		    (setq ispell-pdict-modified-p '(t)) ; dictionary modified!
-		    nil)
-		   ((= char ?m)		; type in what to insert
-		    (process-send-string
-		     ispell-process (concat "*" (read-string "Insert: " word)
-					    "\n"))
-		    (setq ispell-pdict-modified-p '(t))
-		    (cons word nil))
-		   ((and (>= num 0) (< num count))
-		    (if ispell-query-replace-choices ; Query replace flag
-			(list (nth num miss) 'query-replace)
-		      (nth num miss)))
-		   ((= char ?\C-l)
-		    (redraw-display) t)
-		   ((= char ?\C-r)
-		    (save-window-excursion (recursive-edit)) t)
-		   ((= char ?\C-z)
-		    (funcall (key-binding "\C-z"))
-		    t)
-		   (t (ding) t))))))
-	  result))
-    (if (not ispell-keep-choices-win) (bury-buffer ispell-choices-buffer))))
+			    (if (> new-line line)
+				(enlarge-window (- new-line line))
+			      (shrink-window (- line new-line)))
+			    (setq line new-line)))
+		      (select-window (next-window)))))
+	      t)			; reselect from new choices
+	     ((= char ?u)
+	      (process-send-string ispell-process
+				   (concat "*" (downcase word) "\n"))
+	      (setq ispell-pdict-modified-p '(t)) ; dictionary modified!
+	      nil)
+	     ((= char ?m)		; type in what to insert
+	      (process-send-string
+	       ispell-process (concat "*" (read-string "Insert: " word)
+				      "\n"))
+	      (setq ispell-pdict-modified-p '(t))
+	      (cons word nil))
+	     ((and (>= num 0) (< num count))
+	      (if ispell-query-replace-choices ; Query replace flag
+		  (list (nth num miss) 'query-replace)
+		(nth num miss)))
+	     ((= char ?\C-l)
+	      (redraw-display) t)
+	     ((= char ?\C-r)
+	      (save-window-excursion (recursive-edit)) t)
+	     ((= char ?\C-z)
+	      (funcall (key-binding "\C-z"))
+	      t)
+	     (t (ding) t))))))
+    result))
+
 
 
 ;;;###autoload
@@ -1287,7 +1301,9 @@ The variable `ispell-highlight-face' selects the face to use for highlighting."
        (cond
 	((string-match "Lucid" emacs-version)
 	 'ispell-highlight-spelling-error-lucid)
-	((and (string-lessp "19" emacs-version) (featurep 'faces))
+	((and (string-lessp "19" emacs-version)
+	      (featurep 'faces)
+	      window-system)
 	 'ispell-highlight-spelling-error-overlay)
 	(t 'ispell-highlight-spelling-error-generic))))
 
@@ -1295,7 +1311,7 @@ The variable `ispell-highlight-face' selects the face to use for highlighting."
 (defun ispell-overlay-window (height)
   "Create a window covering the top HEIGHT lines of the current window.
 Ensure that the line above point is still visible but otherwise avoid
-scrolling the current window.  Leave the old window selected."
+scrolling the current window.  Leave the new window selected."
   (save-excursion
     (let ((oldot (save-excursion (forward-line -1) (point)))
 	  (top (save-excursion (move-to-window-line height) (point))))
@@ -1523,27 +1539,9 @@ With prefix argument, set the default directory."
 	(message "Spell checking %s..."
 		 (if (and (= reg-start (point-min)) (= reg-end (point-max)))
 		     (buffer-name) "region"))
-	(sit-for 0)
+;Eliminated to keep ispell-message displaying each piece: (sit-for 0)
 	;; must be top level, not in ispell-command-loop for keeping window.
 	(save-window-excursion
-	  (if ispell-keep-choices-win
-	      (let ((ocb (current-buffer))
-		    (window-min-height ispell-choices-win-default-height))
-		(or (eq ocb (window-buffer (selected-window)))
-		    (error
-		     "current buffer is not visible in selected window: %s"
-		     ocb))
-		;; This keeps the default window size when choices window saved
-		(setq ispell-keep-choices-win
-		      ispell-choices-win-default-height)
-		(ispell-overlay-window ispell-choices-win-default-height)
-		(switch-to-buffer (get-buffer-create ispell-choices-buffer))
-		(setq mode-line-format "--  %b  --")
-		(erase-buffer)
-		(select-window (next-window))
-		(or (eq (current-buffer) ocb)
-		    (error "ispell is confused about current buffer!"))
-		(sit-for 0)))
 	  (goto-char reg-start)
 	  (let ((transient-mark-mode nil))
 	    (while (and (not ispell-quit) (< (point) reg-end))
@@ -1654,10 +1652,18 @@ With prefix argument, set the default directory."
 					(ispell-highlight-spelling-error
 					 word-start word-end t))
 				    (sit-for 0)	; update screen display
-				    (setq replace (ispell-command-loop
-						   (car (cdr (cdr poss)))
-						   (car (cdr (cdr (cdr poss))))
-						   (car poss))))
+				    (if ispell-keep-choices-win
+					(setq replace
+					      (ispell-command-loop
+					       (car (cdr (cdr poss)))
+					       (car (cdr (cdr (cdr poss))))
+					       (car poss)))
+				      (save-window-excursion
+					(setq replace
+					      (ispell-command-loop
+					       (car (cdr (cdr poss)))
+					       (car (cdr (cdr (cdr poss))))
+					       (car poss))))))
 				;; protected
 				(if ispell-highlight-p
 				    (ispell-highlight-spelling-error
@@ -1797,7 +1803,6 @@ Standard ispell choices are then available."
   (interactive "P")
   (let ((cursor-location (point))
 	case-fold-search
-	ispell-keep-choices-win
 	(word (ispell-get-word nil "\\*")) ; force "previous-word" processing.
 	start end possibilities replacement)
     (setq start (car (cdr word))
@@ -1823,8 +1828,9 @@ Standard ispell choices are then available."
 	       (progn
 		 (if ispell-highlight-p	; highlight word
 		     (ispell-highlight-spelling-error start end t))
-		 (setq replacement
-		       (ispell-command-loop possibilities nil word)))
+		 (save-window-excursion
+		   (setq replacement
+			 (ispell-command-loop possibilities nil word))))
 	     ;; protected
 	     (if ispell-highlight-p
 		 (ispell-highlight-spelling-error start end))) ; un-highlight
@@ -1869,8 +1875,8 @@ Standard ispell choices are then available."
 	       "^begin [0-9][0-9][0-9] .*\nM.*\nM.*\nM"
 	       ;; Matches shell files (esp. auto-decoding)
 	       "^#! /bin/sh"
-	       ;; Matches difference listing
-	       "diff -c .*\n\\*\\*\\* .*\n--- "
+	       ;; Matches context difference listing
+	       "\\(diff -c .*\\)?\n\\*\\*\\* .*\n--- .*\n\\*\\*\\*\\*\\*\\*\\*\\*\\*\\*\\*\\*\\*\\*\\*"
 	       ;; Matches "----------------- cut here"
 	       "^[-=_]+\\s ?cut here")
 	     "\\|")
@@ -1973,7 +1979,12 @@ you can bind this to the key C-c i in GNUS or mail by adding to
 	;; Skip across text cited from other messages.
 	(while (and (looking-at cite-regexp-start)
 		    (< (point) limit))
-	  (forward-line 1))
+	  (let ((point1 (point)))
+	    (forward-line 1)
+	    ;; If there's no next line, go to the end of this one
+	    ;; so that the loop stops looping.
+	    (if (eq point1 (point))
+		(end-of-line))))
 	(if (< (point) limit)
 	    ;; Check the next batch of lines that *aren't* cited.
 	    (let ((end (save-excursion
@@ -2091,7 +2102,9 @@ Both should not be used to define a buffer-local dictionary."
 	  (setq ispell-buffer-local-name (buffer-name)))
       (let ((end (save-excursion (end-of-line) (point)))
 	    string)
-	(while (re-search-forward " *\\([^ \"]+\\)" end t)
+	;; This used to treat " specially, but that loses for some
+	;; people using them to fake accented letters.
+	(while (re-search-forward " *\\([^ ]+\\)" end t)
 	  (setq string (buffer-substring (match-beginning 1) (match-end 1)))
 	  (process-send-string ispell-process (concat "@" string "\n")))))))
 
