@@ -125,8 +125,8 @@ for `jka-compr-compression-info-list').")
 ;;; compress format; and .gz files, in gzip format.
 (defvar jka-compr-compression-info-list
   ;;[regexp
-  ;; compr-message  compr-prog  compr-discard  compr-args
-  ;; uncomp-message uncomp-prog uncomp-discard uncomp-args
+  ;; compr-message  compr-prog  compr-args
+  ;; uncomp-message uncomp-prog uncomp-args
   ;; can-append auto-mode-flag]
   '(["\\.Z~?\\'"
      "compressing"    "compress"     ("-c")
@@ -139,20 +139,22 @@ for `jka-compr-compression-info-list').")
 
   "List of vectors that describe available compression techniques.
 Each element, which describes a compression technique, is a vector of
-the form [regexp magic compress-name compress-program compress-discard-err
-compress-args uncompress-name uncompress-program uncompress-discard-err
-uncompress-args append-flag extension] where:
+the form [REGEXP COMPRESS-MSG COMPRESS-PROGRAM COMPRESS-ARGS
+UNCOMPRESS-MSG UNCOMPRESS-PROGRAM UNCOMPRESS-ARGS
+APPEND-FLAG EXTENSION], where:
 
    regexp                is a regexp that matches filenames that are
                          compressed with this format
 
+   compress-msg          is the message to issue to the user when doing this
+                         type of compression (nil means no message)
+
    compress-program      is a program that performs this compression
 
    compress-args         is a list of args to pass to the compress program
 
-   uncompress-message    is the message to issue to the user when this
-                         type of uncompression is taking place (nil
-                         means don't issue any message)
+   uncompress-msg        is the message to issue to the user when doing this
+                         type of uncompression (nil means no message)
 
    uncompress-program    is a program that performs this compression
 
@@ -164,11 +166,15 @@ uncompress-args append-flag extension] where:
    auto-mode flag        non-nil means strip the regexp from file names
                          before attempting to set the mode.
 
-Because of the way call-process is defined, discarding the stderr output of
+Because of the way `call-process' is defined, discarding the stderr output of
 a program adds the overhead of starting a shell each time the program is
 invoked.")
 
 
+(defvar jka-compr-file-name-handler-entry
+  nil
+  "The entry in `file-name-handler-alist' used by the jka-compr I/O functions.")
+
 ;;; Functions for accessing the return value of jka-get-compression-info
 (defun jka-compr-info-regexp               (info)  (aref info 0))
 (defun jka-compr-info-compress-message     (info)  (aref info 1))
@@ -231,9 +237,7 @@ based on the filename itself and `jka-compr-compression-info-list'."
 (defun jka-compr-partial-uncompress (prog message args infile beg len)
   "Call program PROG with ARGS args taking input from INFILE.
 Fourth and fifth args, BEG and LEN, specify which part of the output
-to discard.  All output is discarded unless it comes within LEN chars after
-the BEGth char."
-
+to keep: LEN chars starting BEG chars from the beginning."
   (let* ((skip (/ beg jka-compr-dd-blocksize))
 	 (prefix (- beg (* skip jka-compr-dd-blocksize)))
 	 (count (and len (1+ (/ (+ len prefix) jka-compr-dd-blocksize))))
@@ -261,10 +265,13 @@ the BEGth char."
 
       (jka-compr-delete-temp-file err-file))
 
+    ;; Delete the stuff after what we want, if there is any.
     (and
      len
+     (< (+ start prefix len) (point))
      (delete-region (+ start prefix len) (point)))
 
+    ;; Delete the stuff before what we want.
     (delete-region start (+ start prefix))))
 
 
@@ -364,9 +371,6 @@ There should be no more than seven characters after the final `/'")
 
 
 (defun jka-compr-write-region (start end file &optional append visit)
-  "Documented as original."
-  (interactive "r\nFWrite region to file: ")
-
   (let* ((filename (expand-file-name file))
 	 (visit-file (if (stringp visit) (expand-file-name visit) filename))
 	 (info (jka-compr-get-compression-info visit-file)))
@@ -424,7 +428,8 @@ There should be no more than seven characters after the final `/'")
 	     compress-message
 	     (message "%s %s..." compress-message base-name))
 
-	    (write-region start end temp-file t 'dont)
+	    (jka-compr-run-real-handler 'write-region
+					(list start end temp-file t 'dont))
 
 	    (jka-compr-call-process compress-program
 				    (concat compress-message
@@ -435,8 +440,10 @@ There should be no more than seven characters after the final `/'")
 				    compress-args)
 
 	    (set-buffer temp-buffer)
-	    (write-region (point-min) (point-max)
-			  filename (and append can-append) 'dont)
+	    (jka-compr-run-real-handler 'write-region
+					(list (point-min) (point-max)
+					      filename
+					      (and append can-append) 'dont))
 	    (erase-buffer)
 	    (set-buffer cbuf)
 
@@ -462,12 +469,11 @@ There should be no more than seven characters after the final `/'")
 
 	    nil)
 	      
-	(write-region start end filename append visit))))
+	(jka-compr-run-real-handler 'write-region
+				    (list start end filename append visit)))))
 
 
 (defun jka-compr-insert-file-contents (file &optional visit beg end replace)
-  "Documented as original."
-
   (barf-if-buffer-read-only)
 
   (and (or beg end)
@@ -484,7 +490,8 @@ There should be no more than seven characters after the final `/'")
 	      (uncompress-args (jka-compr-info-uncompress-args info))
 	      (base-name (file-name-nondirectory filename))
 	      (notfound nil)
-	      (local-copy (file-local-copy filename))
+	      (local-copy
+	       (jka-compr-run-real-handler 'file-local-copy (list filename)))
 	      local-file
 	      size start)
 
@@ -524,8 +531,15 @@ There should be no more than seven characters after the final `/'")
 						nil
 						uncompress-args))
 		      (setq size (- (point) start))
-		      (goto-char start))
-
+		      (goto-char start)
+		      ;; Run the functions that insert-file-contents would.
+		      (let ((list after-insert-file-functions)
+			    (value size))
+			(while list
+			  (setq value (funcall (car list) size))
+			  (if value
+			      (setq size value))
+			  (setq list (cdr list)))))
 
 		  (error
 		   (if (and (eq (car error-code) 'file-error)
@@ -545,6 +559,7 @@ There should be no more than seven characters after the final `/'")
 	  (and
 	   visit
 	   (progn
+	     (unlock-buffer)
 	     (setq buffer-file-name filename)
 	     (set-visited-file-modtime)))
 	    
@@ -560,12 +575,11 @@ There should be no more than seven characters after the final `/'")
 
 	  (list filename size))
 
-      (insert-file-contents file visit beg end replace))))
+      (jka-compr-run-real-handler 'insert-file-contents
+				  (list file visit beg end replace)))))
 
 
 (defun jka-compr-file-local-copy (file)
-  "Documented as original."
-
   (let* ((filename (expand-file-name file))
 	 (info (jka-compr-get-compression-info filename)))
 
@@ -575,7 +589,8 @@ There should be no more than seven characters after the final `/'")
 	      (uncompress-program (jka-compr-info-uncompress-program info))
 	      (uncompress-args (jka-compr-info-uncompress-args info))
 	      (base-name (file-name-nondirectory filename))
-	      (local-copy (file-local-copy filename))
+	      (local-copy
+	       (jka-compr-run-real-handler 'file-local-copy (list filename)))
 	      (temp-file (jka-compr-make-temp-name t))
 	      (temp-buffer (get-buffer-create " *jka-compr-temp*"))
 	      (notfound nil)
@@ -619,7 +634,7 @@ There should be no more than seven characters after the final `/'")
 
 	  temp-file)
 	    
-      (file-local-copy filename))))
+      (jka-compr-run-real-handler 'file-local-copy (list filename)))))
 
 
 ;;; Support for loading compressed files.
@@ -631,12 +646,8 @@ There should be no more than seven characters after the final `/'")
 
     (unwind-protect
 
-	(progn
-
-	  (setq file-name-handler-alist
-		(cons jka-compr-file-name-handler-entry
-		      file-name-handler-alist))
-
+	(let (inhibit-file-name-operation
+	      inhibit-file-name-handlers)
 	  (or nomessage
 	      (message "Loading %s..." file))
 
@@ -645,38 +656,21 @@ There should be no more than seven characters after the final `/'")
 	  (or nomessage
 	      (message "Loading %s...done." file)))
 
-      (setq file-name-handler-alist
-	    (delq jka-compr-file-name-handler-entry
-		  file-name-handler-alist))
-
       (jka-compr-delete-temp-file local-copy))
 
     t))
-
-
-(defvar jka-compr-file-name-handler-entry
-  nil
-  "The entry in `file-name-handler-alist' used by the jka-compr I/O functions.")
-
+
+(put 'write-region 'jka-compr 'jka-compr-write-region)
+(put 'insert-file-contents 'jka-compr 'jka-compr-insert-file-contents)
+(put 'file-local-copy 'jka-compr 'jka-compr-file-local-copy)
+(put 'load 'jka-compr 'jka-compr-load)
 
 (defun jka-compr-handler (operation &rest args)
-
-  (let ((jka-op (intern-soft (symbol-name operation) jka-compr-op-table))
-	(match-data (match-data)))
-
-    (unwind-protect
-	(progn
-	  (setq file-name-handler-alist
-		(delq jka-compr-file-name-handler-entry
-		      file-name-handler-alist))
-	  (if jka-op
-	      (apply jka-op args)
-	    (jka-compr-run-real-handler operation args)))
-
-      (setq file-name-handler-alist
-	    (cons jka-compr-file-name-handler-entry
-		  file-name-handler-alist))
-      (store-match-data match-data))))
+  (save-match-data
+    (let ((jka-op (get operation 'jka-compr)))
+      (if jka-op
+	  (apply jka-op args)
+	(jka-compr-run-real-handler operation args)))))
 
 ;; If we are given an operation that we don't handle,
 ;; call the Emacs primitive for that operation,
@@ -690,35 +684,6 @@ There should be no more than seven characters after the final `/'")
 	(inhibit-file-name-operation operation))
     (apply operation args)))
 
-  
-(defvar jka-compr-op-table
-  (make-vector 127 0)
-  "Hash table of operations supported by jka-compr.")
-
-
-(defun jka-compr-intern-operation (op)
-  (let ((opsym (intern (symbol-name op) jka-compr-op-table))
-	(jka-fn (intern (concat "jka-compr-" (symbol-name op)))))
-    (fset opsym jka-fn)))
-
-
-(defvar jka-compr-operation-list
-  '(
-    write-region
-    insert-file-contents
-    file-local-copy
-    load
-    )
-  "List of file operations implemented by jka-compr.")
-
-
-(mapcar
- (function
-  (lambda (fn)
-    (jka-compr-intern-operation fn)))
- jka-compr-operation-list)
-
-
 (defun toggle-auto-compression (arg)
   "Toggle automatic file compression and decompression.
 With prefix argument ARG, turn auto compression on if positive, else off.
@@ -769,6 +734,8 @@ This adds entries to `file-name-handler-alist' and `auto-mode-alist'."
   (setq file-name-handler-alist (cons jka-compr-file-name-handler-entry
 				      file-name-handler-alist))
 
+  ;; Make entries in auto-mode-alist so that modes are chosen right
+  ;; according to the file names sans `.gz'.
   (mapcar
    (function (lambda (x)
 	       (and
