@@ -7,7 +7,6 @@
 extern char *strdup (const char *);
 extern int strcasecmp (const char *, const char *);
 */
-extern char *safe_strdup ();
 
 typedef struct _widget_instance
 {
@@ -42,7 +41,7 @@ typedef struct _widget_creation_entry
 } widget_creation_entry;
 
 /* update all other instances of a widget.  Can be used in a callback when
-   a widget has been used by the user */
+   a wiget has been used by the user */
 void
 lw_internal_update_other_instances ();
 
