@@ -5,7 +5,7 @@ This file is part of GNU Emacs.
 
 GNU Emacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
+the Free Software Foundation; either version 1, or (at your option)
 any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
@@ -15,38 +15,23 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* Access the slots of a display-table, according to their purpose.  */
 
-#define DISP_TABLE_P(obj)						    \
-  (CHAR_TABLE_P (obj)							    \
-   && XCHAR_TABLE (obj)->purpose == Qdisplay_table			    \
-   && CHAR_TABLE_EXTRA_SLOTS (XCHAR_TABLE (obj)) == DISP_TABLE_EXTRA_SLOTS)
-
-#define DISP_TABLE_EXTRA_SLOTS 6
-#define DISP_TRUNC_GLYPH(dp) ((dp)->extras[0])
-#define DISP_CONTINUE_GLYPH(dp) ((dp)->extras[1])
-#define DISP_ESCAPE_GLYPH(dp) ((dp)->extras[2])
-#define DISP_CTRL_GLYPH(dp) ((dp)->extras[3])
-#define DISP_INVIS_VECTOR(dp) ((dp)->extras[4])
-#define DISP_BORDER_GLYPH(dp) ((dp)->extras[5])
-
+#define DISP_TABLE_SIZE 261
+#define DISP_TRUNC_GLYPH(dp) ((dp)->contents[256])
+#define DISP_CONTINUE_GLYPH(dp) ((dp)->contents[257])
+#define DISP_ESCAPE_GLYPH(dp) ((dp)->contents[258])
+#define DISP_CTRL_GLYPH(dp) ((dp)->contents[259])
+#define DISP_INVIS_VECTOR(dp) ((dp)->contents[260])
 #define DISP_CHAR_VECTOR(dp, c) ((dp)->contents[c])
 
-/* Defined in window.c.  */
-extern struct Lisp_Char_Table *window_display_table ();
-
-/* Defined in indent.c.  */
-extern struct Lisp_Char_Table *buffer_display_table ();
+extern struct Lisp_Vector *window_display_table ();
 
 /* Display table to use for vectors that don't specify their own.  */
 extern Lisp_Object Vstandard_display_table;
 
-/* This is the `purpose' slot of a display table.  */
-extern Lisp_Object Qdisplay_table;
-
 /* Vector of GLYPH definitions.  Indexed by GLYPH number,
    the contents are a string which is how to output the GLYPH.  */
 extern Lisp_Object Vglyph_table;
@@ -54,21 +39,25 @@ extern Lisp_Object Vglyph_table;
 /* Return the current length of the GLYPH table,
    or 0 if the table isn't currently valid.  */
 #define GLYPH_TABLE_LENGTH  \
-  ((VECTORP (Vglyph_table)) ? XVECTOR (Vglyph_table)->size : 0)
+  ((XTYPE (Vglyph_table) == Lisp_Vector) \
+   ? XVECTOR (Vglyph_table)->size : 0)
 
 /* Return the current base (for indexing) of the GLYPH table,
    or 0 if the table isn't currently valid.  */
 #define GLYPH_TABLE_BASE  \
-  ((VECTORP (Vglyph_table)) ? XVECTOR (Vglyph_table)->contents : 0)
+  ((XTYPE (Vglyph_table) == Lisp_Vector) \
+   ? XVECTOR (Vglyph_table)->contents : 0)
 
 /* Given BASE and LEN returned by the two previous macros,
    return nonzero if the GLYPH code G should be output as a single
    character with code G.  Return zero if G has a string in the table.  */
-#define GLYPH_SIMPLE_P(base,len,g) ((g) >= (len) || !STRINGP (base[g]))
+#define GLYPH_SIMPLE_P(base,len,g)  \
+  ((g) >= (len) || XTYPE (base[g]) != Lisp_String)
 
 /* Given BASE and LEN returned by the two previous macros,
    return nonzero if GLYPH code G is aliased to a different code.  */
-#define GLYPH_ALIAS_P(base,len,g) ((g) < (len) && INTEGERP (base[g]))
+#define GLYPH_ALIAS_P(base,len,g)  \
+  ((g) < (len) && XTYPE (base[g]) == Lisp_Int)
 
 /* Assuming that GLYPH_SIMPLE_P (BASE, LEN, G) is 1,
    return the alias for G.  */
