@@ -1,5 +1,5 @@
 /* Define frame-object for GNU Emacs.
-   Copyright (C) 1993 Free Software Foundation, Inc.
+   Copyright (C) 1993, 1994 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -152,6 +152,11 @@ struct frame
   /* Number of lines of menu bar.  */
   int menu_bar_lines;
 
+#ifdef USE_X_TOOLKIT
+  /* Nonzero means using a menu bar that comes from the X toolkit.  */
+  int external_menu_bar;
+#endif
+
   /* Nonzero if last attempt at redisplay on this frame was preempted.  */
   char display_preempted;
 
@@ -173,7 +178,7 @@ struct frame
      These should probably be considered read-only by everyone except
      FRAME_SAMPLE_VISIBILITY.
 
-     This two are mutually exclusive.  They might both be zero, if the
+     These two are mutually exclusive.  They might both be zero, if the
      frame has been made invisible without an icon.  */
   char visible, iconified;
 
@@ -228,6 +233,13 @@ struct frame
   /* Nonnegative if current redisplay should not do scroll computation
      for lines beyond a certain vpos.  This is the vpos.  */
   int scroll_bottom_vpos;
+
+  /* A vector that records the entire structure of this frame's menu bar.
+     For the format of the data, see extensive comments in xmenu.c.
+     Only the X toolkit version uses this.  */
+  Lisp_Object menu_bar_vector;
+  /* Number of elements in the vector that have meaningful data.  */
+  int menu_bar_items_used;
 };
 
 #ifdef MULTI_FRAME
@@ -253,6 +265,11 @@ typedef struct frame *FRAME_PTR;
 #define FRAME_NEW_HEIGHT(f) (f)->new_height
 #define FRAME_NEW_WIDTH(f) (f)->new_width
 #define FRAME_MENU_BAR_LINES(f) (f)->menu_bar_lines
+#ifdef USE_X_TOOLKIT
+#define FRAME_EXTERNAL_MENU_BAR(f) (f)->external_menu_bar
+#else
+#define FRAME_EXTERNAL_MENU_BAR(f) 0
+#endif
 #define FRAME_CURSOR_X(f) (f)->cursor_x
 #define FRAME_CURSOR_Y(f) (f)->cursor_y
 #define FRAME_VISIBLE_P(f) ((f)->visible != 0)
