@@ -19,12 +19,13 @@ along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 
+#include <config.h>
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/file.h>
 #include <ctype.h>
-#include <config.h>
+#include <errno.h>
 #include "lisp.h"
 
 #ifndef standalone
@@ -49,6 +50,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #endif
 
 #ifdef MSDOS
+#include "msdos.h"
 /* These are redefined (correctly, but differently) in values.h.  */
 #undef INTBITS
 #undef LONGBITS
@@ -58,6 +60,12 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include <math.h>
 #endif /* LISP_FLOAT_TYPE */
 
+#ifndef O_RDONLY
+#define O_RDONLY 0
+#endif
+
+extern int errno;
+
 Lisp_Object Qread_char, Qget_file_char, Qstandard_input, Qcurrent_load_list;
 Lisp_Object Qvariable_documentation, Vvalues, Vstandard_input, Vafter_load_alist;
 Lisp_Object Qascii_character, Qload;
@@ -133,7 +141,18 @@ readchar (readcharfun)
       return c;
     }
   if (EQ (readcharfun, Qget_file_char))
-    return getc (instream);
+    {
+      c = getc (instream);
+#ifdef EINTR
+      /* Interrupted reads have been observed while reading over the network */
+      while (c == EOF && ferror (instream) && errno == EINTR)
+	{
+	  clearerr (instream);
+	  c = getc (instream);
+	}
+#endif
+      return c;
+    }
 
   if (XTYPE (readcharfun) == Lisp_String)
     {
@@ -708,9 +727,10 @@ readevalloop (readcharfun, stream, sourcename, evalfun, printflag)
 
       if (!NILP (Vpurify_flag) && c == '(')
 	{
+	  int count1 = specpdl_ptr - specpdl;
 	  record_unwind_protect (unreadpure, Qnil);
 	  val = read_list (-1, readcharfun);
-	  unbind_to (count + 1, Qnil);
+	  unbind_to (count1, Qnil);
 	}
       else
 	{
@@ -1752,17 +1772,14 @@ defalias (sname, string)
 }
 #endif /* NOTDEF */
 
-/* New replacement for DefIntVar; it ignores the doc string argument
-   on the assumption that make-docfile will handle that.  */
 /* Define an "integer variable"; a symbol whose value is forwarded
  to a C variable of type int.  Sample call: */
   /* DEFVARINT ("indent-tabs-mode", &indent_tabs_mode, "Documentation");  */
 
 void
-defvar_int (namestring, address, doc)
+defvar_int (namestring, address)
      char *namestring;
      int *address;
-     char *doc;
 {
   Lisp_Object sym;
   sym = intern (namestring);
@@ -1773,10 +1790,9 @@ defvar_int (namestring, address, doc)
  NIL if address contains 0 */
 
 void
-defvar_bool (namestring, address, doc)
+defvar_bool (namestring, address)
      char *namestring;
      int *address;
-     char *doc;
 {
   Lisp_Object sym;
   sym = intern (namestring);
@@ -1786,10 +1802,9 @@ defvar_bool (namestring, address, doc)
 /* Similar but define a variable whose value is the Lisp Object stored at address. */
 
 void
-defvar_lisp (namestring, address, doc)
+defvar_lisp (namestring, address)
      char *namestring;
      Lisp_Object *address;
-     char *doc;
 {
   Lisp_Object sym;
   sym = intern (namestring);
@@ -1802,10 +1817,9 @@ defvar_lisp (namestring, address, doc)
    since marking the same slot twice can cause trouble with strings.  */
 
 void
-defvar_lisp_nopro (namestring, address, doc)
+defvar_lisp_nopro (namestring, address)
      char *namestring;
      Lisp_Object *address;
-     char *doc;
 {
   Lisp_Object sym;
   sym = intern (namestring);
