@@ -112,6 +112,10 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #define BSTRING
 
+/* Define this is the compiler understands `volatile'.  */
+#define HAVE_VOLATILE
+
+
 /* subprocesses should be defined if you want to
    have code for asynchronous subprocesses
    (as used in M-x compile and M-x shell).
@@ -209,7 +213,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 :so=\EA\077:se=\EA\027:\
 :ms:mb=\EX\200:md=\EX\010:mk=\EA\161:me=\EA\027:\
 :cl=\EC:ce=\EE:\
-:vb=\EB\140:bl=:"
+:vb=\EB\140:bl=\007:"
 #define fflush internal_flush
 
 /* Define this to a function (Fdowncase, Fupcase) if your file system
@@ -220,3 +224,9 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #define GAP_USE_BCOPY
 #define BCOPY_UPWARD_SAFE 1
 #define BCOPY_DOWNWARD_SAFE 1
+
+/* We need a little extra space, see ../../lisp/loadup.el */
+#define PURESIZE 240000
+
+/* We have (the code to control) a mouse.  */
+#define HAVE_MOUSE
