@@ -1,5 +1,9 @@
 #include "bsd4-2.h"
 
+#ifndef SUNOS4
+#define SUNOS4
+#endif
+
 #if 0  /* This may have been needed for an earlier version of Sun OS 4.
 	  It seems to cause warnings in 4.0.3 and 4.1.  */
 #define O_NDELAY        FNDELAY /* Non-blocking I/O (4.2 style) */
