@@ -1,5 +1,4 @@
-/* Definitions file for GNU Emacs running on UniSoft's UniPlus 5.0
-   Support for this system is not finished; don't expect this to work.
+/* Definitions file for GNU Emacs running on UniSoft's UniPlus 5.2
    Copyright (C) 1985, 1986 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
@@ -35,7 +34,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #define SYSTEM_TYPE "unisoft-unix"
 
-/* NOMULTIPLEJOBS should be defined if your system's shell
+/* nomultiplejobs should be defined if your system's shell
  does not have "job control" (the ability to stop a program,
  run some other program, then continue the first one).  */
 
@@ -46,9 +45,9 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 /* #define INTERRUPT_INPUT */
 
 /* Letter to use in finding device name of first pty,
-  if system supports pty's.  'a' means it is /dev/ptya0  */
+  if system supports pty's.  'p' means it is /dev/ptyp0  */
 
-/* #define FIRST_PTY_LETTER 'a' */
+#define FIRST_PTY_LETTER 'p'
 
 /*
  *	Define HAVE_TERMIO if the system provides sysV-style ioctls
@@ -62,19 +61,19 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
  *	Look in <sys/time.h> for a timeval structure.
  */
 
-/* #define HAVE_TIMEVAL */
+#define HAVE_TIMEVAL
  
 /*
  *	Define HAVE_SELECT if the system supports the `select' system call.
  */
 
-/* #define HAVE_SELECT */
+#define HAVE_SELECT
 
 /*
  *	Define HAVE_PTYS if the system supports pty devices.
  */
 
-/* #define HAVE_PTYS */
+#define HAVE_PTYS
 
 /* Define this macro if system defines a type `union wait'.  */
 
@@ -96,17 +95,18 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 /* #define BSTRING */
 
 /* subprocesses should be defined if you want to
-   have code for asynchronous subprocesses
-   (as used in M-x compile and M-x shell).
-   This is generally OS dependent, and not supported
-   under most USG systems. */
+ have code for asynchronous subprocesses
+ (as used in M-x compile and M-x shell).
+ This is generally OS dependent, and not supported
+ under most USG systems.  It is supported under UniPlus
+ System V Release 2. */
 
-/* #define subprocesses */
+#define subprocesses
 
 /* If your system uses COFF (Common Object File Format) then define the
    preprocessor symbol "COFF". */
 
-/* #define COFF */
+#define COFF
 
 /* define MAIL_USE_FLOCK if the mailer uses flock
    to interlock access to /usr/spool/mail/$USER.
@@ -121,15 +121,9 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* #define CLASH_DETECTION */
 
-/* Define SHORTNAMES if the C compiler can distinguish only
-   short names.  It means that the stuff in ../shortnames
-   must be run to convert the long names to short ones.
+/* Uniplus 5.2 supports long names in C */
 
-   Some USG systems support long names.
-   If yours is one, DO NOT change this file!
-   Do #undef SHORTNAMES in the m- file or in config.h.  */
-
-#define SHORTNAMES
+/* #define SHORTNAMES */
 
 /* We do NOT use the Berkeley (and usg5.2.2) interface to nlist.  */
 
