@@ -215,7 +215,7 @@ matched by parenthesis constructs in the pattern.")
       CHECK_NUMBER (start, 2);
       s = XINT (start);
       if (s < 0 && -s <= len)
-	s = len - s;
+	s = len + s;
       else if (0 > s || s > len)
 	args_out_of_range (string, start);
     }
@@ -348,6 +348,13 @@ scan_buffer (target, start, count, shortage, allow_quit)
   return (start + ((direction == 1 ? 0 : 1)));
 }
 
+int
+find_next_newline_no_quit (from, cnt)
+     register int from, cnt;
+{
+  return scan_buffer ('\n', from, cnt, (int *) 0, 0);
+}
+
 int
 find_next_newline (from, cnt)
      register int from, cnt;
@@ -1175,7 +1182,7 @@ Leaves point at end of replacement text.")
   int some_multiletter_word;
   int some_lowercase;
   int some_uppercase;
-  int some_lowercase_initial;
+  int some_nonuppercase_initial;
   register int c, prevc;
   int inslen;
 
@@ -1205,7 +1212,7 @@ Leaves point at end of replacement text.")
 	 is more than one letter long. */
       some_multiletter_word = 0;
       some_lowercase = 0;
-      some_lowercase_initial = 0;
+      some_nonuppercase_initial = 0;
       some_uppercase = 0;
 
       for (pos = search_regs.start[0]; pos < last; pos++)
@@ -1217,7 +1224,7 @@ Leaves point at end of replacement text.")
 
 	      some_lowercase = 1;
 	      if (SYNTAX (prevc) != Sword)
-		some_lowercase_initial = 1;
+		some_nonuppercase_initial = 1;
 	      else
 		some_multiletter_word = 1;
 	    }
@@ -1229,6 +1236,13 @@ Leaves point at end of replacement text.")
 	      else
 		some_multiletter_word = 1;
 	    }
+	  else
+	    {
+	      /* If the initial is a caseless word constituent,
+		 treat that like a lowercase initial.  */
+	      if (SYNTAX (prevc) != Sword)
+		some_nonuppercase_initial = 1;
+	    }
 
 	  prevc = c;
 	}
@@ -1238,9 +1252,9 @@ Leaves point at end of replacement text.")
       if (! some_lowercase && some_multiletter_word)
 	case_action = all_caps;
       /* Capitalize each word, if the old text has all capitalized words.  */
-      else if (!some_lowercase_initial && some_multiletter_word)
+      else if (!some_nonuppercase_initial && some_multiletter_word)
 	case_action = cap_initial;
-      else if (!some_lowercase_initial && some_uppercase)
+      else if (!some_nonuppercase_initial && some_uppercase)
 	/* Should x -> yz, operating on X, give Yz or YZ?
 	   We'll assume the latter.  */
 	case_action = all_caps;
@@ -1273,7 +1287,7 @@ Leaves point at end of replacement text.")
 		  (Fcurrent_buffer (),
 		   make_number (search_regs.start[0] + offset),
 		   make_number (search_regs.end[0] + offset));
-	      else if (c >= '1' && c <= search_regs.num_regs + '0')
+	      else if (c >= '1' && c <= '9' && c <= search_regs.num_regs + '0')
 		{
 		  if (search_regs.start[c - '0'] >= 1)
 		    Finsert_buffer_substring
