@@ -1,5 +1,5 @@
 /* GNU Emacs routines to deal with syntax tables; also word and list parsing.
-   Copyright (C) 1985, 1987, 1993 Free Software Foundation, Inc.
+   Copyright (C) 1985, 1987, 1993, 1994 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -18,7 +18,7 @@ along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 
-#include "config.h"
+#include <config.h>
 #include <ctype.h>
 #include "lisp.h"
 #include "commands.h"
@@ -84,7 +84,7 @@ find_defun_start (pos)
     return find_start_value;
 
   /* Back up to start of line.  */
-  tem = scan_buffer ('\n', pos, -1, &shortage);
+  tem = scan_buffer ('\n', pos, -1, &shortage, 1);
 
   while (tem > BEGV)
     {
@@ -92,7 +92,7 @@ find_defun_start (pos)
       if (SYNTAX (FETCH_CHAR (tem)) == Sopen)
 	break;
       /* Move to beg of previous line.  */
-      tem = scan_buffer ('\n', tem, -2, &shortage);
+      tem = scan_buffer ('\n', tem, -2, &shortage, 1);
     }
 
   /* Record what we found, for the next try.  */
@@ -197,7 +197,7 @@ unsigned char syntax_spec_code[0400] =
     0377, 0377, 0377, 0377, 0377, 0377, 0377, 0377,
     0377, 0377, 0377, 0377,
 	(char) Scomment, 0377, (char) Sendcomment, 0377,
-    0377, 0377, 0377, 0377, 0377, 0377, 0377, 0377,   /* @, A, ... */
+    (char) Sinherit, 0377, 0377, 0377, 0377, 0377, 0377, 0377,   /* @, A ... */
     0377, 0377, 0377, 0377, 0377, 0377, 0377, 0377,
     0377, 0377, 0377, 0377, 0377, 0377, 0377, (char) Sword,
     0377, 0377, 0377, 0377, (char) Sescape, 0377, 0377, (char) Ssymbol,
@@ -209,9 +209,9 @@ unsigned char syntax_spec_code[0400] =
 
 /* Indexed by syntax code, give the letter that describes it. */
 
-char syntax_code_spec[13] =
+char syntax_code_spec[14] =
   {
-    ' ', '.', 'w', '_', '(', ')', '\'', '\"', '$', '\\', '/', '<', '>'
+    ' ', '.', 'w', '_', '(', ')', '\'', '\"', '$', '\\', '/', '<', '>', '@'
   };
 
 DEFUN ("char-syntax", Fchar_syntax, Schar_syntax, 1, 1, 0,
@@ -223,7 +223,20 @@ are listed in the documentation of `modify-syntax-entry'.")
      Lisp_Object ch;
 {
   CHECK_NUMBER (ch, 0);
-  return make_number (syntax_code_spec[(int) SYNTAX (0xFF & XINT (ch))]);
+  return make_number (syntax_code_spec[(int) SYNTAX (XINT (ch))]);
+}
+
+DEFUN ("matching-paren", Fmatching_paren, Smatching_paren, 1, 1, 0,
+  "Return the matching parenthesis of CHAR, or nil if none.")
+  (ch)
+     Lisp_Object ch;
+{
+  int code;
+  CHECK_NUMBER (ch, 0);
+  code = SYNTAX (XINT (ch));
+  if (code == Sopen || code == Sclose)
+    return make_number (SYNTAX_MATCH (XINT (ch)));
+  return Qnil;
 }
 
 /* This comment supplies the doc string for modify-syntax-entry,
@@ -240,8 +253,9 @@ The first character of S should be one of the following:\n\
   (           open-parenthesis.     )   close-parenthesis.\n\
   \"           string quote.         \\   escape.\n\
   $           paired delimiter.     '   expression quote or prefix operator.\n\
-  <           comment starter.      >   comment ender.\n\                  
-  /           character-quote.\n\
+  <           comment starter.      >   comment ender.\n\
+  /           character-quote.      @   inherit from `standard-syntax-table'.\n\
+\n\
 Only single-character comment start and end sequences are represented thus.\n\
 Two-character sequences are represented as described below.\n\
 The second character of S is the matching parenthesis,\n\
@@ -255,8 +269,8 @@ Defined flags are the characters 1, 2, 3, 4, b, and p.\n\
 \n\
 There can be up to two orthogonal comment sequences. This is to support\n\
 language modes such as C++.  By default, all comment sequences are of style\n\
-a, but you can set the comment sequence style to b (on the second character of a\n\
-comment-start, or the first character of a comment-end sequence) by using\n\
+a, but you can set the comment sequence style to b (on the second character\n\
+of a comment-start, or the first character of a comment-end sequence) using\n\
 this flag:\n\
  b means C is part of comment sequence b.\n\
 \n\
@@ -419,6 +433,8 @@ describe_syntax (value)
       insert_string ("comment"); break;
     case Sendcomment:
       insert_string ("endcomment"); break;
+    case Sinherit:
+      insert_string ("inherit"); break;
     default:
       insert_string ("invalid");
       return;
@@ -596,8 +612,13 @@ between them, return t; otherwise return nil.")
   while (count1 > 0)
     {
       stop = ZV;
-      while (from < stop)
+      do
 	{
+	  if (from == stop)
+	    {
+	      SET_PT (from);
+	      return Qnil;
+	    }
 	  c = FETCH_CHAR (from);
 	  code = SYNTAX (c);
 	  from++;
@@ -605,54 +626,49 @@ between them, return t; otherwise return nil.")
 	  if (from < stop && SYNTAX_COMSTART_FIRST (c)
 	      && SYNTAX_COMSTART_SECOND (FETCH_CHAR (from)))
 	    {
-	      /* we have encountered a comment start sequence and we 
-		 are ignoring all text inside comments. we must record
+	      /* We have encountered a comment start sequence and we 
+		 are ignoring all text inside comments.  We must record
 		 the comment style this sequence begins so that later,
 		 only a comment end of the same style actually ends
-		 the comment section */
+		 the comment section.  */
 	      code = Scomment;
 	      comstyle = SYNTAX_COMMENT_STYLE (FETCH_CHAR (from));
 	      from++;
 	    }
-
-	  if (code == Scomment)
-	    {
-	      while (1)
-		{
-		  if (from == stop)
-		    {
-		      immediate_quit = 0;
-		      SET_PT (from);
-		      return Qnil;
-		    }
-		  c = FETCH_CHAR (from);
-		  if (SYNTAX (c) == Sendcomment
-		      && SYNTAX_COMMENT_STYLE (c) == comstyle)
-		    /* we have encountered a comment end of the same style
-		       as the comment sequence which began this comment
-		       section */
-		    break;
-		  from++;
-		  if (from < stop && SYNTAX_COMEND_FIRST (c)
-		      && SYNTAX_COMEND_SECOND (FETCH_CHAR (from))
-		      && SYNTAX_COMMENT_STYLE (c) == comstyle)
-		    /* we have encountered a comment end of the same style
-		       as the comment sequence which began this comment
-		       section */
-		    { from++; break; }
-		}
-	      /* We have skipped one comment.  */
-	      break;
-	    }
-	  else if (code != Swhitespace && code != Sendcomment)
+	}
+      while (code == Swhitespace || code == Sendcomment);
+      if (code != Scomment)
+	{
+	  immediate_quit = 0;
+	  SET_PT (from - 1);
+	  return Qnil;
+	}
+      /* We're at the start of a comment.  */
+      while (1)
+	{
+	  if (from == stop)
 	    {
 	      immediate_quit = 0;
-	      SET_PT (from - 1);
+	      SET_PT (from);
 	      return Qnil;
 	    }
+	  c = FETCH_CHAR (from);
+	  if (SYNTAX (c) == Sendcomment
+	      && SYNTAX_COMMENT_STYLE (c) == comstyle)
+	    /* we have encountered a comment end of the same style
+	       as the comment sequence which began this comment
+	       section */
+	    break;
+	  from++;
+	  if (from < stop && SYNTAX_COMEND_FIRST (c)
+	      && SYNTAX_COMEND_SECOND (FETCH_CHAR (from))
+	      && SYNTAX_COMMENT_STYLE (c) == comstyle)
+	    /* we have encountered a comment end of the same style
+	       as the comment sequence which began this comment
+	       section */
+	    { from++; break; }
 	}
-
-      /* End of comment reached */
+      /* We have skipped one comment.  */
       count1--;
     }
 
@@ -670,13 +686,15 @@ between them, return t; otherwise return nil.")
 	  c = FETCH_CHAR (from);
 	  code = SYNTAX (c);
 	  comstyle = 0;
+	  if (code == Sendcomment)
+	    comstyle = SYNTAX_COMMENT_STYLE (c);
 	  if (from > stop && SYNTAX_COMEND_SECOND (c)
 	      && SYNTAX_COMEND_FIRST (FETCH_CHAR (from - 1))
 	      && !char_quoted (from - 1))
 	    {
-	      /* we must record the comment style encountered so that
+	      /* We must record the comment style encountered so that
 		 later, we can match only the proper comment begin
-		 sequence of the same style */
+		 sequence of the same style.  */
 	      code = Sendcomment;
 	      comstyle = SYNTAX_COMMENT_STYLE (FETCH_CHAR (from - 1));
 	      from--;
@@ -728,6 +746,7 @@ between them, return t; otherwise return nil.")
 		int comment_end = from;
 		int comstart_pos = 0;
 		int comstart_parity = 0;
+		int scanstart = from - 1;
 
 		/* At beginning of range to scan, we're outside of strings;
 		   that determines quote parity to the comment-end.  */
@@ -746,15 +765,15 @@ between them, return t; otherwise return nil.")
 		      {
 			code = Sendcomment;
 			from--;
+		        c = FETCH_CHAR (from);
 		      }
 			
-		    else if (from > stop && SYNTAX_COMSTART_SECOND (c)
-			     && SYNTAX_COMSTART_FIRST (FETCH_CHAR (from - 1))
-			     && comstyle == SYNTAX_COMMENT_STYLE (c))
-		      {
-			code = Scomment;
-			from--;
-		      }
+		    /* If this char starts a 2-char comment start sequence,
+		       treat it like a 1-char comment starter.  */
+		    if (from < scanstart && SYNTAX_COMSTART_FIRST (c)
+			&& SYNTAX_COMSTART_SECOND (FETCH_CHAR (from + 1))
+			&& comstyle == SYNTAX_COMMENT_STYLE (FETCH_CHAR (from + 1)))
+		      code = Scomment;
 
 		    /* Ignore escaped characters.  */
 		    if (char_quoted (from))
@@ -819,6 +838,8 @@ between them, return t; otherwise return nil.")
 		      from = comment_end;
 		  }
 	      }
+	      /* We have skipped one comment.  */
+	      break;
 	    }
 	  else if ((code != Swhitespace && code != Scomment) || quoted)
 	    {
@@ -926,7 +947,12 @@ scan_lists (from, count, depth, sexpflag)
 	      if (!parse_sexp_ignore_comments) break;
 	      while (1)
 		{
-		  if (from == stop) goto done;
+		  if (from == stop)
+		    {
+		      if (depth == 0)
+			goto done;
+		      goto lose;
+		    }
 		  c = FETCH_CHAR (from);
 		  if (SYNTAX (c) == Sendcomment
 		      && SYNTAX_COMMENT_STYLE (c) == comstyle)
@@ -1013,6 +1039,9 @@ scan_lists (from, count, depth, sexpflag)
 	    from--;
 	  c = FETCH_CHAR (from);
 	  code = SYNTAX (c);
+	  comstyle = 0;
+	  if (code == Sendcomment)
+	    comstyle = SYNTAX_COMMENT_STYLE (c);
 	  if (from > stop && SYNTAX_COMEND_SECOND (c)
 	      && SYNTAX_COMEND_FIRST (FETCH_CHAR (from - 1))
 	      && !char_quoted (from - 1)
@@ -1089,7 +1118,12 @@ scan_lists (from, count, depth, sexpflag)
 		      if (SYNTAX (c = FETCH_CHAR (from)) == Scomment
 			  && SYNTAX_COMMENT_STYLE (c) == comstyle)
 			break;
-		      if (from == stop) goto done;
+		      if (from == stop)
+			{
+			  if (depth == 0)
+			    goto done2;
+			  goto lose;
+			}
 		      from--;
 		      if (SYNTAX_COMSTART_SECOND (c)
 			  && SYNTAX_COMSTART_FIRST (FETCH_CHAR (from))
@@ -1117,6 +1151,7 @@ scan_lists (from, count, depth, sexpflag)
 		int comment_end = from;
 		int comstart_pos = 0;
 		int comstart_parity = 0;
+		int scanstart = from - 1;
 
 		/* At beginning of range to scan, we're outside of strings;
 		   that determines quote parity to the comment-end.  */
@@ -1135,15 +1170,15 @@ scan_lists (from, count, depth, sexpflag)
 		      {
 			code = Sendcomment;
 			from--;
+		        c = FETCH_CHAR (from);
 		      }
 			
-		    else if (from > stop && SYNTAX_COMSTART_SECOND (c)
-			     && SYNTAX_COMSTART_FIRST (FETCH_CHAR (from - 1))
-			     && comstyle == SYNTAX_COMMENT_STYLE (c))
-		      {
-			code = Scomment;
-			from--;
-		      }
+		    /* If this char starts a 2-char comment start sequence,
+		       treat it like a 1-char comment starter.  */
+		    if (from < scanstart && SYNTAX_COMSTART_FIRST (c)
+			&& SYNTAX_COMSTART_SECOND (FETCH_CHAR (from + 1))
+			&& comstyle == SYNTAX_COMMENT_STYLE (FETCH_CHAR (from + 1)))
+		      code = Scomment;
 
 		    /* Ignore escaped characters.  */
 		    if (char_quoted (from))
@@ -1699,6 +1734,7 @@ syms_of_syntax ()
   defsubr (&Scopy_syntax_table);
   defsubr (&Sset_syntax_table);
   defsubr (&Schar_syntax);
+  defsubr (&Smatching_paren);
   defsubr (&Smodify_syntax_entry);
   defsubr (&Sdescribe_syntax);
 
