@@ -18,9 +18,9 @@ along with GNU Emacs; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 
+#include <config.h>
 #include <stdio.h>
 #include <ctype.h>
-#include <config.h>
 #include "termchar.h"
 #include "termopts.h"
 #include "cm.h"
@@ -285,6 +285,8 @@ FRAME_PTR updating_frame;
 static int system_uses_terminfo;
 
 char *tparam ();
+
+extern char *tgetstr ();
 
 ring_bell ()
 {
@@ -1254,7 +1256,6 @@ term_get_fkeys (address)
      refusing to run at all on such a terminal.  */
 
   extern Lisp_Object Fidentity ();
-  static Lisp_Object term_get_fkeys_1 ();
   term_get_fkeys_arg = address;
   internal_condition_case (term_get_fkeys_1, Qerror, Fidentity);
 }
@@ -1262,7 +1263,6 @@ term_get_fkeys (address)
 static Lisp_Object
 term_get_fkeys_1 ()
 {
-  extern char *tgetstr ();
   int i;
 
   char **address = term_get_fkeys_arg;
@@ -1363,8 +1363,6 @@ term_init (terminal_type)
   register char *p;
   int status;
 
-  extern char *tgetstr ();
-
   Wcm_clear ();
   dont_calculate_costs = 0;
 
@@ -1513,7 +1511,7 @@ to do `unset TERMCAP' (C-shell: `unsetenv TERMCAP') as well.\n",
      If that fails, we can't use standout mode at all.  */
   if (TS_end_standout_mode == 0)
     {
-      char *s = tgetstr ("me");
+      char *s = tgetstr ("me", address);
       if (s != 0)
 	TS_end_standout_mode = s;
       else
