@@ -19,8 +19,8 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* Emacs 19 face widget ported by Fred Pierresteguy */
 
-#include <stdio.h>
 #include <config.h>
+#include <stdio.h>
 #include "lisp.h"
 #include "xterm.h"
 
@@ -880,7 +880,6 @@ EmacsFrameSetCharSize (widget, columns, rows)
   int ac = 0;
   
   if (columns < 3) columns = 3;  /* no way buddy */
-  if (rows < 3) rows = 3;
 
   check_frame_size (f, &rows, &columns);
   f->display.x->vertical_scroll_bar_extra
@@ -894,6 +893,9 @@ EmacsFrameSetCharSize (widget, columns, rows)
     {
       int hdelta = pixel_height - ew->core.height;
       int column_widget_height = f->display.x->column_widget->core.height;
+      int old_left = f->display.x->widget->core.x;
+      int old_top = f->display.x->widget->core.y;
+
       XawPanedSetRefigureMode (f->display.x->column_widget, False);
 
       ac = 0;
@@ -907,6 +909,10 @@ EmacsFrameSetCharSize (widget, columns, rows)
       XtSetValues (f->display.x->column_widget, al, ac);
 
       XawPanedSetRefigureMode (f->display.x->column_widget, True);
+
+      /* These seem to get clobbered.  I don't know why. - rms.  */
+      f->display.x->widget->core.x = old_left;
+      f->display.x->widget->core.y = old_top;
     }
 
   /* We've set {FRAME,PIXEL}_{WIDTH,HEIGHT} to the values we hope to
