@@ -1,5 +1,5 @@
 /* X Selection processing for emacs
-   Copyright (C) 1993 Free Software Foundation.
+   Copyright (C) 1993, 1994 Free Software Foundation.
 
 This file is part of GNU Emacs.
 
@@ -23,7 +23,7 @@ x_reply_selection_request  */
 
 /* Rewritten by jwz */
 
-#include "config.h"
+#include <config.h>
 #include "lisp.h"
 #if 0
 #include <stdio.h>	/* termhooks.h needs this */
@@ -73,13 +73,14 @@ Lisp_Object Vx_sent_selection_hooks;
 unsigned long last_event_timestamp;
 
 /* This is an association list whose elements are of the form
-     ( selection-name selection-value selection-timestamp )
-   selection-name is a lisp symbol, whose name is the name of an X Atom.
-   selection-value is the value that emacs owns for that selection.
+     ( SELECTION-NAME SELECTION-VALUE SELECTION-TIMESTAMP FRAME)
+   SELECTION-NAME is a lisp symbol, whose name is the name of an X Atom.
+   SELECTION-VALUE is the value that emacs owns for that selection.
      It may be any kind of Lisp object.
-   selection-timestamp is the time at which emacs began owning this selection,
+   SELECTION-TIMESTAMP is the time at which emacs began owning this selection,
      as a cons of two 16-bit numbers (making a 32 bit time.)
-   If there is an entry in this alist, then it can be assumed that emacs owns
+   FRAME is the frame for which we made the selection.
+   If there is an entry in this alist, then it can be assumed that Emacs owns
     that selection.
    The only (eq) parts of this list that are visible from Lisp are the
     selection-values.
@@ -98,7 +99,6 @@ Lisp_Object Vselection_converter_alist;
    we give up on it.  This is in milliseconds (0 = no timeout.)
  */
 int x_selection_timeout;
-
 
 /* Utility functions */
 
@@ -106,11 +106,6 @@ static void lisp_data_to_selection_data ();
 static Lisp_Object selection_data_to_lisp_data ();
 static Lisp_Object x_get_window_property_as_lisp_data ();
 
-static int expect_property_change ();
-static void wait_for_property_change ();
-static void unexpect_property_change ();
-static int waiting_for_other_props_on_window ();
-
 /* This converts a Lisp symbol to a server Atom, avoiding a server 
    roundtrip whenever possible.  */
 
@@ -242,11 +237,7 @@ x_own_selection (selection_name, selection_value)
      Lisp_Object selection_name, selection_value;
 {
   Display *display = x_current_display;
-#ifdef X_TOOLKIT
-  Window selecting_window = XtWindow (selected_screen->display.x->edit_widget);
-#else
   Window selecting_window = FRAME_X_WINDOW (selected_frame);
-#endif
   Time time = last_event_timestamp;
   Atom selection_atom;
 
@@ -254,7 +245,10 @@ x_own_selection (selection_name, selection_value)
   selection_atom = symbol_to_x_atom (display, selection_name);
 
   BLOCK_INPUT;
+  x_catch_errors ();
   XSetSelectionOwner (display, selection_atom, selecting_window, time);
+  x_check_errors ("Can't set selection: %s");
+  x_uncatch_errors ();
   UNBLOCK_INPUT;
 
   /* Now update the local cache */
@@ -266,7 +260,8 @@ x_own_selection (selection_name, selection_value)
     selection_time = long_to_cons ((unsigned long) time);
     selection_data = Fcons (selection_name,
 			    Fcons (selection_value,
-				   Fcons (selection_time, Qnil)));
+				   Fcons (selection_time,
+					  Fcons (Fselected_frame (), Qnil))));
     prev_value = assq_no_quit (selection_name, Vselection_alist);
 
     Vselection_alist = Fcons (selection_data, Vselection_alist);
@@ -328,9 +323,11 @@ x_get_local_selection (selection_symbol, target_type)
   else if (CONSP (target_type)
 	   && XCONS (target_type)->car == QMULTIPLE)
     {
-      Lisp_Object pairs = XCONS (target_type)->cdr;
-      int size = XVECTOR (pairs)->size;
+      Lisp_Object pairs;
+      int size;
       int i;
+      pairs = XCONS (target_type)->cdr;
+      size = XVECTOR (pairs)->size;
       /* If the target is MULTIPLE, then target_type looks like
 	  (MULTIPLE . [[SELECTION1 TARGET1] [SELECTION2 TARGET2] ... ])
 	 We modify the second element of each pair in the vector and
@@ -338,7 +335,8 @@ x_get_local_selection (selection_symbol, target_type)
        */
       for (i = 0; i < size; i++)
 	{
-	  Lisp_Object pair = XVECTOR (pairs)->contents [i];
+	  Lisp_Object pair;
+	  pair = XVECTOR (pairs)->contents [i];
 	  XVECTOR (pair)->contents [1]
 	    = x_get_local_selection (XVECTOR (pair)->contents [0],
 				     XVECTOR (pair)->contents [1]);
@@ -415,8 +413,9 @@ x_decline_selection_request (event)
   reply.property = None;
 
   BLOCK_INPUT;
-  (void) XSendEvent (reply.display, reply.requestor, False, 0L,
-		     (XEvent *) &reply);
+  XSendEvent (reply.display, reply.requestor, False, 0L,
+	      (XEvent *) &reply);
+  XFlushQueue ();
   UNBLOCK_INPUT;
 }
 
@@ -437,6 +436,37 @@ x_selection_request_lisp_error (ignore)
   return Qnil;
 }
 
+
+/* This stuff is so that INCR selections are reentrant (that is, so we can
+   be servicing multiple INCR selection requests simultaneously.)  I haven't
+   actually tested that yet.  */
+
+/* Keep a list of the property changes that are awaited.  */
+
+struct prop_location
+{
+  int identifier;
+  Display *display;
+  Window window;
+  Atom property;
+  int desired_state;
+  int arrived;
+  struct prop_location *next;
+};
+
+static struct prop_location *expect_property_change ();
+static void wait_for_property_change ();
+static void unexpect_property_change ();
+static int waiting_for_other_props_on_window ();
+
+static int prop_location_identifier;
+
+static Lisp_Object property_change_reply;
+
+static struct prop_location *property_change_reply_object;
+
+static struct prop_location *property_change_wait_list;
+
 /* Send the reply to a selection request event EVENT.
    TYPE is the type of selection data requested.
    DATA and SIZE describe the data to send, already converted.
@@ -492,7 +522,7 @@ x_reply_selection_request (event, format, data, size, type)
   else
     {
       /* Send an INCR selection.  */
-      int prop_id;
+      struct prop_location *wait_object;
 
       BLOCK_INPUT;
 
@@ -501,8 +531,8 @@ x_reply_selection_request (event, format, data, size, type)
 #if 0
       fprintf (stderr, "\nINCR %d\n", bytes_remaining);
 #endif
-      prop_id = expect_property_change (display, window, reply.property,
-					PropertyDelete);
+      wait_object = expect_property_change (display, window, reply.property,
+					    PropertyDelete);
 
       XChangeProperty (display, window, reply.property, Xatom_INCR,
 		       32, PropModeReplace, (unsigned char *)
@@ -515,7 +545,7 @@ x_reply_selection_request (event, format, data, size, type)
 
       /* First, wait for the requestor to ack by deleting the property.
 	 This can run random lisp code (process handlers) or signal.  */
-      wait_for_property_change (prop_id);
+      wait_for_property_change (wait_object);
 
       while (bytes_remaining)
 	{
@@ -525,8 +555,9 @@ x_reply_selection_request (event, format, data, size, type)
 
 	  BLOCK_INPUT;
 
-	  prop_id = expect_property_change (display, window, reply.property,
-					    PropertyDelete);
+	  wait_object
+	    = expect_property_change (display, window, reply.property,
+				      PropertyDelete);
 #if 0
 	  fprintf (stderr,"  INCR adding %d\n", i);
 #endif
@@ -541,7 +572,7 @@ x_reply_selection_request (event, format, data, size, type)
 	  /* Now wait for the requestor to ack this chunk by deleting the
 	     property.	 This can run random lisp code or signal.
 	   */
-	  wait_for_property_change (prop_id);
+	  wait_for_property_change (wait_object);
 	}
       /* Now write a zero-length chunk to the property to tell the requestor
 	 that we're done.  */
@@ -567,14 +598,19 @@ x_handle_selection_request (event)
      struct input_event *event;
 {
   struct gcpro gcpro1, gcpro2, gcpro3;
-  Lisp_Object local_selection_data = Qnil;
+  Lisp_Object local_selection_data;
   Lisp_Object selection_symbol;
-  Lisp_Object target_symbol = Qnil;
-  Lisp_Object converted_selection = Qnil;
+  Lisp_Object target_symbol;
+  Lisp_Object converted_selection;
   Time local_selection_time;
-  Lisp_Object successful_p = Qnil;
+  Lisp_Object successful_p;
   int count;
 
+  local_selection_data = Qnil;
+  target_symbol = Qnil;
+  converted_selection = Qnil;
+  successful_p = Qnil;
+
   GCPRO3 (local_selection_data, converted_selection, target_symbol);
 
   selection_symbol = x_atom_to_symbol (SELECTION_EVENT_DISPLAY (event),
@@ -649,7 +685,8 @@ x_handle_selection_request (event)
 
   /* Let random lisp code notice that the selection has been asked for.  */
   {
-    Lisp_Object rest = Vx_sent_selection_hooks;
+    Lisp_Object rest;
+    rest = Vx_sent_selection_hooks;
     if (!EQ (rest, Qunbound))
       for (; CONSP (rest); rest = Fcdr (rest))
 	call3 (Fcar (rest), selection_symbol, target_symbol, successful_p);
@@ -708,50 +745,72 @@ x_handle_selection_clear (event)
   /* Let random lisp code notice that the selection has been stolen.  */
 
   {
-    Lisp_Object rest = Vx_lost_selection_hooks;
+    Lisp_Object rest;
+    rest = Vx_lost_selection_hooks;
     if (!EQ (rest, Qunbound))
-      for (; CONSP (rest); rest = Fcdr (rest))
-	call1 (Fcar (rest), selection_symbol);
+      {
+	for (; CONSP (rest); rest = Fcdr (rest))
+	  call1 (Fcar (rest), selection_symbol);
+	prepare_menu_bars ();
+	redisplay_preserve_echo_area ();
+      }
   }
 }
 
-
-/* This stuff is so that INCR selections are reentrant (that is, so we can
-   be servicing multiple INCR selection requests simultaneously.)  I haven't
-   actually tested that yet.  */
+/* Clear all selections that were made from frame F.
+   We do this when about to delete a frame.  */
 
-static int prop_location_identifier;
-
-static Lisp_Object property_change_reply;
-static int property_change_reply_identifier;
-
-/* Keep a list of the property changes that are awaited.  */
-
-struct prop_location
+void
+x_clear_frame_selections (f)
+     FRAME_PTR f;
 {
-  int identifier;
-  Display *display;
-  Window window;
-  Atom property;
-  int desired_state;
-  struct prop_location *next;
-};
+  Lisp_Object frame;
+  Lisp_Object rest;
 
-static struct prop_location *property_change_wait_list;
+  XSET (frame, Lisp_Frame, f);
 
-static int
-property_deleted_p (identifier)
-     void *identifier;
-{
-  struct prop_location *rest = property_change_wait_list;
-  while (rest)
-    if (rest->identifier == (int) identifier)
-      return 0;
-    else
-      rest = rest->next;
-  return 1;
+  /* Otherwise, we're really honest and truly being told to drop it.
+     Don't use Fdelq as that may QUIT;.  */
+
+  while (!NILP (Vselection_alist)
+	 && EQ (frame, Fcar (Fcdr (Fcdr (Fcdr (Fcar (Vselection_alist)))))))
+    {
+      /* Let random Lisp code notice that the selection has been stolen.  */
+      Lisp_Object hooks, selection_symbol;
+
+      hooks = Vx_lost_selection_hooks;
+      selection_symbol = Fcar (Vselection_alist);
+
+      if (!EQ (hooks, Qunbound))
+	{
+	  for (; CONSP (hooks); hooks = Fcdr (hooks))
+	    call1 (Fcar (hooks), selection_symbol);
+	  redisplay_preserve_echo_area ();
+	}
+
+      Vselection_alist = Fcdr (Vselection_alist);
+    }
+
+  for (rest = Vselection_alist; !NILP (rest); rest = Fcdr (rest))
+    if (EQ (frame, Fcar (Fcdr (Fcdr (Fcdr (Fcar (XCONS (rest)->cdr)))))))
+      {
+	/* Let random Lisp code notice that the selection has been stolen.  */
+	Lisp_Object hooks, selection_symbol;
+
+	hooks = Vx_lost_selection_hooks;
+	selection_symbol = Fcar (XCONS (rest)->cdr);
+
+	if (!EQ (hooks, Qunbound))
+	  {
+	    for (; CONSP (hooks); hooks = Fcdr (hooks))
+	      call1 (Fcar (hooks), selection_symbol);
+	    redisplay_preserve_echo_area ();
+	  }
+	XCONS (rest)->cdr = Fcdr (XCONS (rest)->cdr);
+	break;
+      }
 }
-
+
 /* Nonzero if any properties for DISPLAY and WINDOW
    are on the list of what we are waiting for.  */
 
@@ -774,7 +833,7 @@ waiting_for_other_props_on_window (display, window)
    The return value is a number that uniquely identifies
    this awaited property change.  */
 
-static int
+static struct prop_location *
 expect_property_change (display, window, property, state)
      Display *display;
      Window window;
@@ -789,21 +848,22 @@ expect_property_change (display, window, property, state)
   pl->property = property;
   pl->desired_state = state;
   pl->next = property_change_wait_list;
+  pl->arrived = 0;
   property_change_wait_list = pl;
-  return pl->identifier;
+  return pl;
 }
 
 /* Delete an entry from the list of property changes we are waiting for.
    IDENTIFIER is the number that uniquely identifies the entry.  */
 
 static void
-unexpect_property_change (identifier)
-     int identifier;
+unexpect_property_change (location)
+     struct prop_location *location;
 {
   struct prop_location *prev = 0, *rest = property_change_wait_list;
   while (rest)
     {
-      if (rest->identifier == identifier)
+      if (rest == location)
 	{
 	  if (prev)
 	    prev->next = rest->next;
@@ -823,30 +883,37 @@ static Lisp_Object
 wait_for_property_change_unwind (identifierval)
      Lisp_Object identifierval;
 {
-  unexpect_property_change (XFASTINT (identifierval));
+  unexpect_property_change (XPNTR (identifierval));
 }
 
 /* Actually wait for a property change.
    IDENTIFIER should be the value that expect_property_change returned.  */
 
 static void
-wait_for_property_change (identifier)
+wait_for_property_change (location)
+     struct prop_location *location;
 {
   int secs, usecs;
   int count = specpdl_ptr - specpdl;
+  Lisp_Object tem;
+
+  XSET (tem, Lisp_Cons, location);
 
   /* Make sure to do unexpect_property_change if we quit or err.  */
-  record_unwind_protect (wait_for_property_change_unwind,
-			 make_number (identifier));
+  record_unwind_protect (wait_for_property_change_unwind, tem);
 
   XCONS (property_change_reply)->car = Qnil;
-  property_change_reply_identifier = identifier;
-  secs = x_selection_timeout / 1000;
-  usecs = (x_selection_timeout % 1000) * 1000;
-  wait_reading_process_input (secs, usecs, property_change_reply, 0);
 
-  if (NILP (XCONS (property_change_reply)->car))
-    error ("timed out waiting for property-notify event");
+  if (! location->arrived)
+    {
+      property_change_reply_object = location;
+      secs = x_selection_timeout / 1000;
+      usecs = (x_selection_timeout % 1000) * 1000;
+      wait_reading_process_input (secs, usecs, property_change_reply, 0);
+
+      if (NILP (XCONS (property_change_reply)->car))
+	error ("timed out waiting for property-notify event");
+    }
 
   unbind_to (count, Qnil);
 }
@@ -873,9 +940,11 @@ x_handle_property_notify (event)
 		   ->name->data);
 #endif
 
+	  rest->arrived = 1;
+
 	  /* If this is the one wait_for_property_change is waiting for,
 	     tell it to wake up.  */
-	  if (rest->identifier == property_change_reply_identifier)
+	  if (rest == property_change_reply_object)
 	    XCONS (property_change_reply)->car = Qt;
 
 	  if (prev)
@@ -961,11 +1030,7 @@ x_get_foreign_selection (selection_symbol, target_type)
      Lisp_Object selection_symbol, target_type;
 {
   Display *display = x_current_display;
-#ifdef X_TOOLKIT
-  Window requestor_window = XtWindow (selected_screen->display.x->edit_widget);
-#else
   Window requestor_window = FRAME_X_WINDOW (selected_frame);
-#endif
   Time requestor_time = last_event_timestamp;
   Atom target_property = Xatom_EMACS_TMP;
   Atom selection_atom = symbol_to_x_atom (display, selection_symbol);
@@ -978,6 +1043,7 @@ x_get_foreign_selection (selection_symbol, target_type)
     type_atom = symbol_to_x_atom (display, target_type);
 
   BLOCK_INPUT;
+  x_catch_errors ();
   XConvertSelection (display, selection_atom, type_atom, target_property,
 		     requestor_window, requestor_time);
   XFlushQueue ();
@@ -986,6 +1052,7 @@ x_get_foreign_selection (selection_symbol, target_type)
   reading_selection_window = requestor_window;
   reading_which_selection = selection_atom;
   XCONS (reading_selection_reply)->car = Qnil;
+  x_start_queuing_selection_requests ();
   UNBLOCK_INPUT;
 
   /* This allows quits.  Also, don't wait forever.  */
@@ -993,6 +1060,12 @@ x_get_foreign_selection (selection_symbol, target_type)
   usecs = (x_selection_timeout % 1000) * 1000;
   wait_reading_process_input (secs, usecs, reading_selection_reply, 0);
 
+  BLOCK_INPUT;
+  x_check_errors ("Cannot get selection: %s");
+  x_uncatch_errors ();
+  x_stop_queuing_selection_requests ();
+  UNBLOCK_INPUT;
+
   if (NILP (XCONS (reading_selection_reply)->car))
     error ("timed out waiting for reply from selection owner");
 
@@ -1100,7 +1173,7 @@ receive_incremental_selection (display, window, property, target_type,
      int *format_ret;
 {
   int offset = 0;
-  int prop_id;
+  struct prop_location *wait_object;
   *size_bytes_ret = min_size_bytes;
   *data_ret = (unsigned char *) xmalloc (*size_bytes_ret);
 #if 0
@@ -1118,8 +1191,8 @@ receive_incremental_selection (display, window, property, target_type,
   BLOCK_INPUT;
   XSelectInput (display, window, STANDARD_EVENT_SET | PropertyChangeMask);
   XDeleteProperty (display, window, property);
-  prop_id = expect_property_change (display, window, property,
-				    PropertyNewValue);
+  wait_object = expect_property_change (display, window, property,
+					PropertyNewValue);
   XFlushQueue ();
   UNBLOCK_INPUT;
 
@@ -1127,7 +1200,7 @@ receive_incremental_selection (display, window, property, target_type,
     {
       unsigned char *tmp_data;
       int tmp_size_bytes;
-      wait_for_property_change (prop_id);
+      wait_for_property_change (wait_object);
       /* expect it again immediately, because x_get_window_property may
 	 .. no it wont, I dont get it.
 	 .. Ok, I get it now, the Xt code that implements INCR is broken.
@@ -1143,15 +1216,15 @@ receive_incremental_selection (display, window, property, target_type,
 #endif
 	  if (! waiting_for_other_props_on_window (display, window))
 	    XSelectInput (display, window, STANDARD_EVENT_SET);
-	  unexpect_property_change (prop_id);
+	  unexpect_property_change (wait_object);
 	  if (tmp_data) xfree (tmp_data);
 	  break;
 	}
 
       BLOCK_INPUT;
       XDeleteProperty (display, window, property);
-      prop_id = expect_property_change (display, window, property,
-					PropertyNewValue);
+      wait_object = expect_property_change (display, window, property,
+					    PropertyNewValue);
       XFlushQueue ();
       UNBLOCK_INPUT;
 
@@ -1167,7 +1240,7 @@ receive_incremental_selection (display, window, property, target_type,
 	  *size_bytes_ret = offset + tmp_size_bytes;
 	  *data_ret = (unsigned char *) xrealloc (*data_ret, *size_bytes_ret);
 	}
-      memcpy ((*data_ret) + offset, tmp_data, tmp_size_bytes);
+      bcopy (tmp_data, (*data_ret) + offset, tmp_size_bytes);
       offset += tmp_size_bytes;
       xfree (tmp_data);
     }
@@ -1565,6 +1638,7 @@ anything that the functions on `selection-converter-alist' know about.")
   (selection_name, selection_value)
      Lisp_Object selection_name, selection_value;
 {
+  check_x ();
   CHECK_SYMBOL (selection_name, 0);
   if (NILP (selection_value)) error ("selection-value may not be nil.");
   x_own_selection (selection_name, selection_value);
@@ -1588,6 +1662,7 @@ TYPE is the type of data desired, typically `STRING'.")
   Lisp_Object val = Qnil;
   struct gcpro gcpro1, gcpro2;
   GCPRO2 (target_type, val); /* we store newly consed data into these */
+  check_x ();
   CHECK_SYMBOL (selection_symbol, 0);
 
 #if 0 /* #### MULTIPLE doesn't work yet */
@@ -1636,6 +1711,7 @@ Disowning it means there is no such selection.")
   Atom selection_atom;
   XSelectionClearEvent event;
 
+  check_x ();
   CHECK_SYMBOL (selection, 0);
   if (NILP (time))
     timestamp = last_event_timestamp;
@@ -1656,9 +1732,9 @@ Disowning it means there is no such selection.")
      the selection owner to None.  The NCD server does, the MIT Sun4 server
      doesn't.  So we synthesize one; this means we might get two, but
      that's ok, because the second one won't have any effect.  */
-  event.display = display;
-  event.selection = selection_atom;
-  event.time = timestamp;
+  SELECTION_EVENT_DISPLAY (&event) = display;
+  SELECTION_EVENT_SELECTION (&event) = selection_atom;
+  SELECTION_EVENT_TIME (&event) = timestamp;
   x_handle_selection_clear (&event);
 
   return Qt;
@@ -1696,6 +1772,7 @@ and t is the same as `SECONDARY'.)")
   (selection)
      Lisp_Object selection;
 {
+  check_x ();
   CHECK_SYMBOL (selection, 0);
   if (EQ (selection, Qnil)) selection = QPRIMARY;
   if (EQ (selection, Qt)) selection = QSECONDARY;
@@ -1719,6 +1796,7 @@ and t is the same as `SECONDARY'.)")
   Window owner;
   Atom atom;
   Display *dpy = x_current_display;
+  check_x ();
   CHECK_SYMBOL (selection, 0);
   if (!NILP (Fx_selection_owner_p (selection)))
     return Qt;
@@ -1789,6 +1867,7 @@ DEFUN ("x-get-cut-buffer-internal", Fx_get_cut_buffer_internal,
   unsigned long size;
   Lisp_Object ret;
 
+  check_x ();
   CHECK_CUT_BUFFER (buffer, 0);
   buffer_atom = symbol_to_x_atom (display, buffer);
 
@@ -1823,6 +1902,7 @@ DEFUN ("x-store-cut-buffer-internal", Fx_store_cut_buffer_internal,
   int max_bytes = SELECTION_QUANTUM (display);
   if (max_bytes > MAX_SELECTION_QUANTUM) max_bytes = MAX_SELECTION_QUANTUM;
 
+  check_x ();
   CHECK_CUT_BUFFER (buffer, 0);
   CHECK_STRING (string, 0);
   buffer_atom = symbol_to_x_atom (display, buffer);
@@ -1867,6 +1947,7 @@ positive means move values forward, negative means backward.")
   Window window = RootWindow (display, 0); /* Cut buffers are on screen 0 */
   Atom props [8];
 
+  check_x ();
   CHECK_NUMBER (n, 0);
   if (XINT (n) == 0) return n;
   if (! cut_buffers_initialized) initialize_cut_buffers (display, window);
